Package: BrainNetTest
Type: Package
Title: Hypothesis Testing for Populations of Brain Networks
Version: 0.2.1
Authors@R: c(
    person("Maximiliano", "Martino",
           email = "maxii.martino@gmail.com",
           role  = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-2437-4387")),
    person("Daniel", "Fraiman",
           email = "dfraiman@udesa.edu.ar",
           role  = "aut",
           comment = c(ORCID = "0000-0002-0482-9137")))
Description: Non-parametric hypothesis testing for populations of brain
    networks represented as graphs, following the L1-distance ANOVA
    framework of Fraiman and Fraiman (2018) <doi:10.1038/s41598-018-21688-0>.
    The package builds on this nonparametric graph-comparison framework,
    extending it with procedures for edge-level inference and identification
    of the specific connections driving group differences. In particular, it
    provides utilities to compute central (mean) graphs, pairwise Manhattan
    distances between adjacency matrices, the group test statistic T, and a
    fast permutation procedure to identify the critical edges that drive
    between-group differences. Helper functions to generate synthetic
    community-structured graphs and to visualise brain networks with
    communities are also included.
License: MIT + file LICENSE
URL: https://github.com/mmaximiliano/BrainNetTest
BugReports: https://github.com/mmaximiliano/BrainNetTest/issues
Encoding: UTF-8
Depends: R (>= 4.0.0)
Imports: stats, graphics, igraph
Suggests: knitr, rmarkdown, testthat (>= 3.1.7)
RoxygenNote: 7.3.3
VignetteBuilder: knitr
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-06-04 20:31:12 UTC; maxim
Author: Maximiliano Martino [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-2437-4387>),
  Daniel Fraiman [aut] (ORCID: <https://orcid.org/0000-0002-0482-9137>)
Maintainer: Maximiliano Martino <maxii.martino@gmail.com>
Repository: CRAN
Date/Publication: 2026-06-05 05:20:02 UTC
Built: R 4.6.0; ; 2026-06-05 05:30:44 UTC; unix

# Generated by roxygen2: do not edit by hand

export(compute_central_graph)
export(compute_distance)
export(compute_edge_frequencies)
export(compute_edge_pvalues)
export(compute_test_statistic)
export(generate_category_graphs)
export(generate_community_graph)
export(generate_random_graph)
export(get_critical_nodes)
export(identify_critical_links)
export(plot_critical_edges)
export(rank_edges)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(stats,chisq.test)
importFrom(stats,fisher.test)
importFrom(stats,p.adjust)
importFrom(stats,prop.test)
importFrom(stats,rbinom)
importFrom(stats,runif)

# R/identify_critical_links_naive.R
# Naive (brute-force) reference implementation of identify_critical_links.
# ---------------------------------------------------------------------------
# This function recomputes the test statistic T from scratch after every edge
# removal and regenerates the full permutation null at each step.  Its
# complexity is O(K * B * |E| * m) -- the "naive" baseline described in the
# paper.  It is kept internally for benchmarking and as a correctness
# reference for the optimised `identify_critical_links()`; it is NOT exported.
#
#' @noRd
#' @keywords internal
identify_critical_links_naive <- function(populations,
                                          alpha          = 0.05,
                                          method         = "fisher",
                                          adjust_method  = "none",
                                          batch_size     = 1,
                                          n_permutations = 1000,
                                          a              = 1,
                                          seed           = NULL) {

  if (!is.null(seed)) set.seed(seed)
  if (!is.list(populations) || length(populations) < 2)
    stop("`populations` must be a list with at least 2 groups.")
  if (batch_size  < 1L) stop("`batch_size` must be >= 1.")
  if (n_permutations < 1L) stop("`n_permutations` must be >= 1.")

  ## 0. Metadata ------------------------------------------------------------
  Npop <- sapply(populations, length)
  m    <- length(populations)
  n    <- sum(Npop)

  ## 1. Rank edges by marginal p-value (same as optimised) -----------------
  freq       <- compute_edge_frequencies(populations)
  edge_pvals <- compute_edge_pvalues(freq$edge_counts, Npop,
                                     method        = method,
                                     adjust_method = adjust_method)
  edge_df    <- rank_edges(edge_pvals)
  n_edges    <- nrow(edge_df)
  if (n_edges == 0L)
    return(list(critical_edges       = NULL,
                edges_removed        = list(),
                modified_populations = populations))

  ## Helper: permutation p-value for the current populations ----------------
  .permutation_pvalue <- function(pops, Npop_vec, n_total, n_perm, a_val) {
    T_obs      <- compute_test_statistic(pops, a = a_val)
    all_graphs <- unlist(pops, recursive = FALSE)
    T_perm     <- numeric(n_perm)
    for (b in seq_len(n_perm)) {
      perm_idx  <- sample.int(n_total)
      cumN      <- c(0L, cumsum(Npop_vec))
      perm_pops <- lapply(seq_along(Npop_vec), function(k)
        all_graphs[perm_idx[(cumN[k] + 1L):cumN[k + 1L]]])
      names(perm_pops) <- names(pops)
      T_perm[b] <- compute_test_statistic(perm_pops, a = a_val)
    }
    mean(T_perm < T_obs)
  }

  ## 2. Global test ---------------------------------------------------------
  initial_p <- .permutation_pvalue(populations, Npop, n, n_permutations, a)
  if (initial_p > alpha) {
    warning("Initial test is not significant (p = ",
            round(initial_p, 4),
            "). Populations may be identical or differences too small to detect.")
    return(list(critical_edges       = NULL,
                edges_removed        = list(),
                modified_populations = populations))
  }

  ## 3. Iterative edge removal (naive: full recomputation each step) ------
  edges_removed <- list()
  k_removed     <- 0L
  continue      <- TRUE

  while (continue && k_removed < n_edges) {
    batch_end <- min(k_removed + batch_size, n_edges)
    idx_batch <- (k_removed + 1L):batch_end

    ## Physically remove edges from all graphs -----------------------------
    for (idx in idx_batch) {
      i <- edge_df$node1[idx]; j <- edge_df$node2[idx]
      for (pop in seq_along(populations))
        for (g in seq_len(Npop[pop])) {
          populations[[pop]][[g]][i, j] <- 0
          populations[[pop]][[g]][j, i] <- 0
        }
      edges_removed[[length(edges_removed) + 1L]] <- c(i, j)
    }
    k_removed <- batch_end

    ## Recompute T and permutation null FROM SCRATCH -----------------------
    p_k      <- .permutation_pvalue(populations, Npop, n, n_permutations, a)
    continue <- (p_k <= alpha)
  }

  critical_edges <- if (k_removed > 0)
                      edge_df[seq_len(k_removed), ]
                    else
                      NULL

  list(critical_edges       = critical_edges,
       edges_removed        = edges_removed,
       modified_populations = populations)
}

# R/generate_category_graphs.R

#' Generate a Set of Graphs with Similar Community Structures for a Category
#'
#' This function generates a list of adjacency matrices representing brain networks
#' belonging to the same category (e.g., Control group). The generated graphs have
#' similar community structures but vary slightly in their intra-community and
#' inter-community connection probabilities to reflect natural variability.
#'
#' @param n_graphs An integer specifying the number of graphs to generate. Default is 10.
#' @param n_nodes An integer specifying the total number of nodes (brain regions). Default is 100.
#' @param n_communities An integer specifying the number of communities. Default is 4.
#' @param community_sizes An integer vector specifying the sizes of each community.
#'   If NULL, communities are of equal size. Default is NULL.
#' @param base_intra_prob A numeric value between 0 and 1, or a numeric vector
#'   of length \code{n_communities}, specifying the base probability of an edge
#'   existing between nodes within the same community. If a vector, each element
#'   sets the base probability for the corresponding community. Default is 0.8.
#' @param base_inter_prob A numeric value between 0 and 1 specifying the base probability
#'   of an edge existing between nodes from different communities. Default is 0.2.
#' @param intra_prob_variation A numeric value specifying the maximum variation to apply
#'   to the intra-community probability for each graph. Default is 0.05.
#' @param inter_prob_variation A numeric value specifying the maximum variation to apply
#'   to the inter-community probability for each graph. Default is 0.05.
#' @param seed An optional integer for setting the random seed to ensure reproducibility.
#'   Default is NULL.
#'
#' @return A list of symmetric binary adjacency matrices with no self-loops, representing
#'   brain networks with similar community structures.
#' @export
#'
#' @importFrom stats runif
#' @examples
#' # Generate a set of 5 graphs for the Control category
#' control_graphs <- generate_category_graphs(n_graphs = 5, n_nodes = 100, n_communities = 4,
#'                                            base_intra_prob = 0.8, base_inter_prob = 0.2)
generate_category_graphs <- function(n_graphs = 10, n_nodes = 100, n_communities = 4, community_sizes = NULL,
                                     base_intra_prob = 0.8, base_inter_prob = 0.2,
                                     intra_prob_variation = 0.05, inter_prob_variation = 0.05, seed = NULL) {
  if (!is.null(seed)) {
    set.seed(seed)
  }
  
  if (!is.numeric(n_graphs) || length(n_graphs) != 1 || n_graphs <= 0) {
    stop("n_graphs must be a positive integer.")
  }
  n_graphs <- as.integer(n_graphs)
  
  # Validate probabilities and variations
  if (!is.numeric(base_intra_prob) || any(base_intra_prob < 0) || any(base_intra_prob > 1)) {
    stop("base_intra_prob must be between 0 and 1.")
  }
  if (!is.numeric(base_inter_prob) || base_inter_prob < 0 || base_inter_prob > 1) {
    stop("base_inter_prob must be between 0 and 1.")
  }
  if (!is.numeric(intra_prob_variation) || intra_prob_variation < 0 || intra_prob_variation > 1) {
    stop("intra_prob_variation must be between 0 and 1.")
  }
  if (!is.numeric(inter_prob_variation) || inter_prob_variation < 0 || inter_prob_variation > 1) {
    stop("inter_prob_variation must be between 0 and 1.")
  }
  
  # Generate a base community assignment
  if (is.null(community_sizes)) {
    # Equal community sizes
    base_size <- n_nodes %/% n_communities
    remainder <- n_nodes %% n_communities
    community_sizes <- rep(base_size, n_communities)
    if (remainder > 0) {
      community_sizes[1:remainder] <- community_sizes[1:remainder] + 1
    }
  } else {
    if (length(community_sizes) != n_communities) {
      stop("Length of community_sizes must equal n_communities.")
    }
    if (sum(community_sizes) != n_nodes) {
      stop("Sum of community_sizes must equal n_nodes.")
    }
  }
  
  # List to store generated graphs
  graph_list <- vector("list", n_graphs)
  
  for (g in 1:n_graphs) {
    # Apply slight variations to probabilities
    intra_prob <- base_intra_prob + runif(length(base_intra_prob),
                                          -intra_prob_variation, intra_prob_variation)
    inter_prob <- base_inter_prob + runif(1, -inter_prob_variation, inter_prob_variation)
    
    # Ensure probabilities remain within [0, 1]
    intra_prob <- pmin(pmax(intra_prob, 0), 1)
    inter_prob <- min(max(inter_prob, 0), 1)
    
    # Generate the graph using the base community assignments
    G <- generate_community_graph(n_nodes = n_nodes, n_communities = n_communities,
                                  community_sizes = community_sizes,
                                  intra_prob = intra_prob, inter_prob = inter_prob)
    
    graph_list[[g]] <- G
  }
  
  return(graph_list)
}

# tests/testthat/test-compute_edge_pvalues.R

# Helper: build a 2-pop edge_counts array from raw matrices
.make_counts <- function(pop_counts_list) {
  n_nodes <- nrow(pop_counts_list[[1]])
  arr <- array(0, dim = c(n_nodes, n_nodes, length(pop_counts_list)))
  for (k in seq_along(pop_counts_list))
    arr[, , k] <- pop_counts_list[[k]]
  arr
}

test_that("compute_edge_pvalues returns a symmetric matrix with valid p-values", {
  set.seed(1)
  ctrl <- generate_category_graphs(n_graphs = 10, n_nodes = 6, n_communities = 2,
    base_intra_prob = 0.8, base_inter_prob = 0.2, seed = 1)
  dis  <- generate_category_graphs(n_graphs = 10, n_nodes = 6, n_communities = 2,
    base_intra_prob = 0.4, base_inter_prob = 0.6, seed = 2)
  pops <- list(A = ctrl, B = dis)
  freq <- compute_edge_frequencies(pops)

  pvals <- compute_edge_pvalues(freq$edge_counts, sapply(pops, length))
  expect_equal(dim(pvals), c(6, 6))
  expect_true(isSymmetric(pvals))
  expect_true(all(pvals >= 0 & pvals <= 1))
  # Diagonal not meaningful but kept at 1
  expect_equal(diag(pvals), rep(1, 6))
})

test_that("compute_edge_pvalues: identical populations yield p >= alpha mostly", {
  set.seed(1)
  g <- generate_category_graphs(n_graphs = 20, n_nodes = 5, n_communities = 1,
    base_intra_prob = 0.5, base_inter_prob = 0.5, seed = 1)
  pops <- list(A = g[1:10], B = g[11:20])
  freq <- compute_edge_frequencies(pops)
  pvals <- compute_edge_pvalues(freq$edge_counts, sapply(pops, length))
  # With 10+10 samples drawn from the same process, no edge should be strongly
  # significant; all p-values must be valid probabilities.
  expect_true(all(pvals >= 0 & pvals <= 1))
})

test_that("compute_edge_pvalues supports all three test methods", {
  set.seed(2)
  ctrl <- generate_category_graphs(n_graphs = 10, n_nodes = 5, n_communities = 1,
    base_intra_prob = 0.9, base_inter_prob = 0.9, seed = 1)
  dis  <- generate_category_graphs(n_graphs = 10, n_nodes = 5, n_communities = 1,
    base_intra_prob = 0.1, base_inter_prob = 0.1, seed = 2)
  pops <- list(A = ctrl, B = dis)
  freq <- compute_edge_frequencies(pops)
  N <- sapply(pops, length)

  p_fisher <- compute_edge_pvalues(freq$edge_counts, N, method = "fisher")
  p_chisq  <- suppressWarnings(
    compute_edge_pvalues(freq$edge_counts, N, method = "chi.squared"))
  p_prop   <- suppressWarnings(
    compute_edge_pvalues(freq$edge_counts, N, method = "prop"))

  for (pv in list(p_fisher, p_chisq, p_prop)) {
    expect_equal(dim(pv), c(5, 5))
    expect_true(isSymmetric(pv))
    expect_true(all(pv >= 0 & pv <= 1))
  }
})

test_that("compute_edge_pvalues: invalid method throws", {
  counts <- .make_counts(list(matrix(c(0, 1, 1, 0), nrow = 2),
                              matrix(c(0, 0, 0, 0), nrow = 2)))
  expect_error(compute_edge_pvalues(counts, c(1, 1), method = "tukey"),
               "Invalid method")
})

test_that("compute_edge_pvalues: prop method requires exactly two populations", {
  counts <- array(0, dim = c(3, 3, 3))   # 3 populations
  expect_error(compute_edge_pvalues(counts, c(1, 1, 1), method = "prop"),
               "only implemented for two populations")
})

test_that("compute_edge_pvalues: adjust_method BH reduces significance", {
  set.seed(3)
  ctrl <- generate_category_graphs(n_graphs = 10, n_nodes = 5, n_communities = 1,
    base_intra_prob = 0.8, base_inter_prob = 0.8, seed = 1)
  dis  <- generate_category_graphs(n_graphs = 10, n_nodes = 5, n_communities = 1,
    base_intra_prob = 0.2, base_inter_prob = 0.2, seed = 2)
  pops <- list(A = ctrl, B = dis)
  freq <- compute_edge_frequencies(pops)
  N    <- sapply(pops, length)

  p_raw <- compute_edge_pvalues(freq$edge_counts, N, adjust_method = "none")
  p_bh  <- compute_edge_pvalues(freq$edge_counts, N, adjust_method = "BH")

  # Adjusted p-values are never smaller than raw p-values (BH monotone) on the
  # upper triangle, which is what downstream code consumes.
  ut <- upper.tri(p_raw)
  expect_true(all(p_bh[ut] >= p_raw[ut] - 1e-12))
  # Valid probabilities
  expect_true(all(p_bh >= 0 & p_bh <= 1))
})

test_that("compute_edge_pvalues returns a symmetric matrix for all adjust_methods", {
  # Regression test for a bug where the BH (and other) adjustment was written
  # incorrectly to the lower triangle, producing an asymmetric result.
  set.seed(7)
  ctrl <- generate_category_graphs(n_graphs = 6, n_nodes = 6, n_communities = 2,
    base_intra_prob = 0.9, base_inter_prob = 0.2, seed = 1)
  dis  <- generate_category_graphs(n_graphs = 6, n_nodes = 6, n_communities = 2,
    base_intra_prob = 0.3, base_inter_prob = 0.6, seed = 2)
  pops <- list(A = ctrl, B = dis)
  freq <- compute_edge_frequencies(pops)
  N    <- sapply(pops, length)

  for (adj in c("none", "BH", "bonferroni", "holm", "BY")) {
    p <- compute_edge_pvalues(freq$edge_counts, N, adjust_method = adj)
    expect_true(isSymmetric(p),
                info = paste("Non-symmetric p-value matrix for adjust_method =", adj))
    expect_true(all(p >= 0 & p <= 1))
  }
})

test_that("compute_edge_pvalues: prop with all-zero or all-present edges yields p = 1", {
  # One edge always absent in both groups
  n <- 4
  ec <- array(0, dim = c(n, n, 2))
  N  <- c(5, 5)
  pv <- compute_edge_pvalues(ec, N, method = "prop")
  expect_equal(unname(pv[1, 2]), 1.0)
  # One edge always present in both groups
  ec2 <- ec; ec2[1, 2, ] <- N; ec2[2, 1, ] <- N
  pv2 <- compute_edge_pvalues(ec2, N, method = "prop")
  expect_equal(unname(pv2[1, 2]), 1.0)
})

# tests/testthat/test-brainnettest.R

library(testthat)
library(BrainNetTest)

# ---------------------------------------------------------------------------
# compute_central_graph
# ---------------------------------------------------------------------------
test_that("compute_central_graph averages adjacency matrices", {
  G1 <- matrix(c(0, 1, 0, 1, 0, 1, 0, 1, 0), nrow = 3)
  G2 <- matrix(c(0, 0, 1, 0, 0, 1, 1, 0, 0), nrow = 3)
  central <- compute_central_graph(list(G1, G2))
  expect_equal(central, (G1 + G2) / 2)
  # Entries are the fraction of graphs containing each edge
  expect_true(all(central >= 0 & central <= 1))
})

test_that("compute_central_graph validates inputs", {
  expect_error(compute_central_graph(list()), "empty")
  expect_error(compute_central_graph(list(1, 2)), "must be matrices")
  M1 <- matrix(0, 3, 3); M2 <- matrix(0, 4, 4)
  expect_error(compute_central_graph(list(M1, M2)),
               "same dimensions")
})

test_that("compute_central_graph of a singleton returns that graph", {
  G <- matrix(c(0, 1, 1, 0), 2, 2)
  expect_equal(compute_central_graph(list(G)), G)
})

# ---------------------------------------------------------------------------
# compute_distance
# ---------------------------------------------------------------------------
test_that("compute_distance returns the L1 norm", {
  G <- matrix(c(0, 1, 0, 1, 0, 1, 0, 1, 0), nrow = 3)
  M <- matrix(c(0, 0, 1, 0, 0, 1, 1, 0, 0), nrow = 3)
  expect_equal(compute_distance(G, M), sum(abs(G - M)))
})

test_that("compute_distance(G, G) == 0", {
  set.seed(1)
  G <- generate_random_graph(n_nodes = 6, edge_prob = 0.3)
  expect_equal(compute_distance(G, G), 0)
})

test_that("compute_distance is symmetric", {
  set.seed(1)
  G1 <- generate_random_graph(6, 0.3)
  G2 <- generate_random_graph(6, 0.4)
  expect_equal(compute_distance(G1, G2), compute_distance(G2, G1))
})

test_that("compute_distance validates inputs", {
  expect_error(compute_distance("a", matrix(0, 2, 2)), "matrices")
  expect_error(compute_distance(matrix(0, 2, 2), list()), "matrices")
  expect_error(compute_distance(matrix(0, 2, 2), matrix(0, 3, 3)),
               "same dimensions")
})

# ---------------------------------------------------------------------------
# compute_test_statistic
# ---------------------------------------------------------------------------
test_that("compute_test_statistic returns a finite numeric", {
  set.seed(1)
  ctrl <- replicate(5, generate_random_graph(6, 0.3), simplify = FALSE)
  dis  <- replicate(5, generate_random_graph(6, 0.6), simplify = FALSE)
  T_value <- compute_test_statistic(list(A = ctrl, B = dis), a = 1)
  expect_type(T_value, "double")
  expect_length(T_value, 1L)
  expect_true(is.finite(T_value))
})

test_that("compute_test_statistic is invariant to population order", {
  set.seed(2)
  A <- replicate(6, generate_random_graph(6, 0.3), simplify = FALSE)
  B <- replicate(6, generate_random_graph(6, 0.5), simplify = FALSE)
  T1 <- compute_test_statistic(list(A = A, B = B), a = 1)
  T2 <- compute_test_statistic(list(B = B, A = A), a = 1)
  expect_equal(unname(T1), unname(T2))
})

test_that("compute_test_statistic scales as 1 / a", {
  set.seed(3)
  A <- replicate(5, generate_random_graph(5, 0.3), simplify = FALSE)
  B <- replicate(5, generate_random_graph(5, 0.5), simplify = FALSE)
  T_a1 <- compute_test_statistic(list(A = A, B = B), a = 1)
  T_a2 <- compute_test_statistic(list(A = A, B = B), a = 2)
  expect_equal(T_a2, T_a1 / 2)
})

test_that("compute_test_statistic validates inputs", {
  expect_error(compute_test_statistic(list()), "non-empty list")
  expect_error(compute_test_statistic("bad"),  "non-empty list")
  # Each population must have at least 2 graphs
  G <- matrix(0, 3, 3)
  expect_error(
    compute_test_statistic(list(A = list(G), B = list(G, G))),
    "at least two graphs")
})



# ---------------------------------------------------------------------------
# Multi-population (m >= 3) support
# ---------------------------------------------------------------------------
test_that("pipeline works end-to-end for m = 3 populations", {
  skip_on_cran()
  set.seed(1)
  A <- generate_category_graphs(8, 10, 2, base_intra_prob = 0.9,
                                base_inter_prob = 0.1, seed = 1)
  B <- generate_category_graphs(8, 10, 2, base_intra_prob = 0.5,
                                base_inter_prob = 0.5, seed = 2)
  C <- generate_category_graphs(8, 10, 2, base_intra_prob = 0.1,
                                base_inter_prob = 0.9, seed = 3)
  pops <- list(A = A, B = B, C = C)

  # compute_test_statistic accepts m = 3
  T_val <- compute_test_statistic(pops, a = 1)
  expect_true(is.finite(T_val))

  # compute_edge_frequencies returns a 3-slice array
  fr <- compute_edge_frequencies(pops)
  expect_equal(dim(fr$edge_counts)[3], 3L)

  # Fisher and chi-squared both admit m = 3 via the m x 2 contingency table
  N <- sapply(pops, length)
  for (mm in c("fisher", "chi.squared")) {
    p <- suppressWarnings(compute_edge_pvalues(fr$edge_counts, N, method = mm))
    expect_true(isSymmetric(p), info = paste("method =", mm))
    expect_true(all(p >= 0 & p <= 1))
  }

  # prop.test is explicitly restricted to 2 populations by design
  expect_error(compute_edge_pvalues(fr$edge_counts, N, method = "prop"),
               "two populations")

  # Full critical-edge pipeline works for m = 3
  res <- suppressWarnings(identify_critical_links(
    pops, n_permutations = 200, seed = 42))
  expect_true(is.list(res))
  expect_true(all(c("critical_edges", "edges_removed",
                    "modified_populations") %in% names(res)))
  expect_equal(length(res$modified_populations), 3L)
})

test_that("compute_test_statistic is invariant to population order for m = 3", {
  set.seed(11)
  A <- replicate(5, generate_random_graph(6, 0.3), simplify = FALSE)
  B <- replicate(5, generate_random_graph(6, 0.5), simplify = FALSE)
  C <- replicate(5, generate_random_graph(6, 0.7), simplify = FALSE)
  T1 <- compute_test_statistic(list(A = A, B = B, C = C), a = 1)
  T2 <- compute_test_statistic(list(C = C, A = A, B = B), a = 1)
  expect_equal(unname(T1), unname(T2))
})

test_that("get_critical_nodes returns correct structure without labels", {
  skip_on_cran()

  set.seed(42)
  ctrl <- generate_category_graphs(n_graphs = 15, n_nodes = 10,
    n_communities = 2, base_intra_prob = 0.8, base_inter_prob = 0.2, seed = 1)
  dis  <- generate_category_graphs(n_graphs = 15, n_nodes = 10,
    n_communities = 2, base_intra_prob = 0.5, base_inter_prob = 0.5, seed = 2)
  pops <- list(Control = ctrl, Patient = dis)

  result <- identify_critical_links(pops, alpha = 0.05, method = "fisher",
    n_permutations = 200, seed = 99)

  nodes_df <- get_critical_nodes(result)

  expect_true(is.data.frame(nodes_df))
  expect_true("node" %in% names(nodes_df))
  expect_true("critical_degree" %in% names(nodes_df))
  expect_false("label" %in% names(nodes_df))

  # Every node must appear in at least one critical edge
  expect_true(all(nodes_df$critical_degree >= 1))

  # Nodes must be a subset of nodes in critical_edges
  ce <- result$critical_edges
  all_edge_nodes <- unique(c(ce$node1, ce$node2))
  expect_true(all(nodes_df$node %in% all_edge_nodes))
  expect_equal(sort(nodes_df$node), sort(all_edge_nodes))

  # Sorted by critical_degree descending
  expect_true(all(diff(nodes_df$critical_degree) <= 0))
})


test_that("get_critical_nodes includes labels when provided", {
  skip_on_cran()

  set.seed(42)
  ctrl <- generate_category_graphs(n_graphs = 15, n_nodes = 10,
    n_communities = 2, base_intra_prob = 0.8, base_inter_prob = 0.2, seed = 1)
  dis  <- generate_category_graphs(n_graphs = 15, n_nodes = 10,
    n_communities = 2, base_intra_prob = 0.5, base_inter_prob = 0.5, seed = 2)
  pops <- list(Control = ctrl, Patient = dis)

  result <- identify_critical_links(pops, alpha = 0.05, method = "fisher",
    n_permutations = 200, seed = 99)

  labels <- paste0("R", 1:10)
  nodes_df <- get_critical_nodes(result, node_labels = labels)

  expect_true("label" %in% names(nodes_df))
  # Column order: node, label, critical_degree
  expect_equal(names(nodes_df), c("node", "label", "critical_degree"))
  # Labels must match
  for (i in seq_len(nrow(nodes_df))) {
    expect_equal(nodes_df$label[i], labels[nodes_df$node[i]])
  }
})


test_that("get_critical_nodes handles NULL critical_edges", {
  # Simulate a result where no differences were found
  result <- list(
    critical_edges       = NULL,
    edges_removed        = list(),
    modified_populations = list()
  )

  nodes_df <- get_critical_nodes(result)
  expect_true(is.data.frame(nodes_df))
  expect_equal(nrow(nodes_df), 0)
  expect_true("node" %in% names(nodes_df))
  expect_true("critical_degree" %in% names(nodes_df))

  # Also with labels
  nodes_df2 <- get_critical_nodes(result, node_labels = c("A", "B", "C"))
  expect_equal(nrow(nodes_df2), 0)
  expect_true("label" %in% names(nodes_df2))
})


test_that("get_critical_nodes validates inputs", {
  # Not a list
  expect_error(get_critical_nodes("not_a_list"), "identify_critical_links")

  # Missing critical_edges component
  expect_error(get_critical_nodes(list(a = 1)), "identify_critical_links")

  # node_labels not character
  result <- list(critical_edges = data.frame(node1 = 1, node2 = 2, p_value = 0.01))
  expect_error(get_critical_nodes(result, node_labels = 1:5), "character")

  # node_labels too short
  expect_error(get_critical_nodes(result, node_labels = "A"), "length 1")
})


test_that("get_critical_nodes critical_degree sums correctly", {
  # Manual example: edges (1,2), (1,3), (2,3)
  ce <- data.frame(node1 = c(1L, 1L, 2L), node2 = c(2L, 3L, 3L),
                   p_value = c(0.001, 0.01, 0.02))
  result <- list(critical_edges = ce, edges_removed = list(), modified_populations = list())

  nodes_df <- get_critical_nodes(result)

  expect_equal(nrow(nodes_df), 3)
  # Node 1 appears in edges (1,2) and (1,3) → degree 2
  expect_equal(nodes_df$critical_degree[nodes_df$node == 1], 2L)
  # Node 2 appears in edges (1,2) and (2,3) → degree 2
  expect_equal(nodes_df$critical_degree[nodes_df$node == 2], 2L)
  # Node 3 appears in edges (1,3) and (2,3) → degree 2
  expect_equal(nodes_df$critical_degree[nodes_df$node == 3], 2L)
})

# tests/testthat/test-generate_category_graphs.R

test_that("generate_category_graphs works correctly", {
  set.seed(123)
  n_graphs <- 5
  n_nodes <- 50
  n_communities <- 3
  graphs <- generate_category_graphs(n_graphs = n_graphs, n_nodes = n_nodes, n_communities = n_communities,
                                     base_intra_prob = 0.8, base_inter_prob = 0.2,
                                     intra_prob_variation = 0.05, inter_prob_variation = 0.05)
  
  expect_equal(length(graphs), n_graphs)
  
  for (G in graphs) {
    expect_equal(dim(G), c(n_nodes, n_nodes))
    expect_true(isSymmetric(G))
    expect_equal(diag(G), rep(0, n_nodes))
  }
  
  # Check that community assignments are consistent across graphs
  community_sizes <- NULL
  if (is.null(community_sizes)) {
    base_size <- n_nodes %/% n_communities
    remainder <- n_nodes %% n_communities
    community_sizes <- rep(base_size, n_communities)
    if (remainder > 0) {
      community_sizes[1:remainder] <- community_sizes[1:remainder] + 1
    }
  }
  community_assignments <- rep(1:n_communities, times = community_sizes)
  
  # Check that intra-community edges are generally more frequent than inter-community edges
  intra_edge_counts <- numeric(n_graphs)
  inter_edge_counts <- numeric(n_graphs)
  
  for (i in seq_along(graphs)) {
    G <- graphs[[i]]
    intra_edges <- 0
    inter_edges <- 0
    for (c in 1:n_communities) {
      nodes_in_c <- which(community_assignments == c)
      intra_edges <- intra_edges + sum(G[nodes_in_c, nodes_in_c])
    }
    inter_edges <- sum(G) - intra_edges
    intra_edge_counts[i] <- intra_edges
    inter_edge_counts[i] <- inter_edges
    expect_true(intra_edges > inter_edges)
  }

  # Expect some variation in edge counts due to probability variations
  expect_true(sd(intra_edge_counts) > 0)
  expect_true(sd(inter_edge_counts) > 0)
})

test_that("generate_category_graphs is reproducible with seed", {
  g1 <- generate_category_graphs(n_graphs = 5, n_nodes = 10, n_communities = 2,
    base_intra_prob = 0.7, base_inter_prob = 0.2, seed = 42)
  g2 <- generate_category_graphs(n_graphs = 5, n_nodes = 10, n_communities = 2,
    base_intra_prob = 0.7, base_inter_prob = 0.2, seed = 42)
  expect_identical(g1, g2)
})

test_that("generate_category_graphs validates inputs", {
  expect_error(
    generate_category_graphs(n_graphs = 0, n_nodes = 10, n_communities = 2),
    "positive integer")
  expect_error(
    generate_category_graphs(n_graphs = 3, n_nodes = 10, n_communities = 2,
                             base_intra_prob = 1.2),
    "between 0 and 1")
  expect_error(
    generate_category_graphs(n_graphs = 3, n_nodes = 10, n_communities = 2,
                             base_inter_prob = -0.1),
    "between 0 and 1")
  expect_error(
    generate_category_graphs(n_graphs = 3, n_nodes = 10, n_communities = 2,
                             intra_prob_variation = 2),
    "between 0 and 1")
  expect_error(
    generate_category_graphs(n_graphs = 3, n_nodes = 10, n_communities = 2,
                             inter_prob_variation = -0.5),
    "between 0 and 1")
  expect_error(
    generate_category_graphs(n_graphs = 3, n_nodes = 10, n_communities = 3,
                             community_sizes = c(5, 5)),
    "Length of community_sizes")
  expect_error(
    generate_category_graphs(n_graphs = 3, n_nodes = 10, n_communities = 2,
                             community_sizes = c(3, 3)),
    "Sum of community_sizes")
})

test_that("generate_category_graphs supports vector base_intra_prob", {
  set.seed(1)
  g <- generate_category_graphs(n_graphs = 4, n_nodes = 20, n_communities = 2,
                                base_intra_prob = c(0.9, 0.1),
                                base_inter_prob = 0.2,
                                intra_prob_variation = 0.02,
                                inter_prob_variation = 0.02,
                                seed = 1)
  expect_equal(length(g), 4)
  for (G in g) {
    expect_equal(dim(G), c(20, 20))
    expect_true(isSymmetric(G))
    expect_equal(diag(G), rep(0, 20))
  }
})

# tests/testthat/test-plot_critical_edges.R

make_populations <- function(seed = 1) {
  set.seed(seed)
  control <- generate_category_graphs(
    n_graphs = 10, n_nodes = 12, n_communities = 3,
    community_sizes = c(4, 4, 4),
    base_intra_prob = rep(0.8, 3), base_inter_prob = 0.1,
    intra_prob_variation = 0.02, inter_prob_variation = 0.02, seed = seed)
  patient <- generate_category_graphs(
    n_graphs = 10, n_nodes = 12, n_communities = 3,
    community_sizes = c(4, 4, 4),
    base_intra_prob = c(0.4, 0.8, 0.8), base_inter_prob = 0.1,
    intra_prob_variation = 0.02, inter_prob_variation = 0.02,
    seed = seed + 1)
  list(Control = control, Patient = patient)
}

test_that("plot_critical_edges runs without errors on a typical workflow", {
  skip_on_cran()
  skip_if_not_installed("igraph")
  populations <- make_populations()
  result <- identify_critical_links(populations, alpha = 0.05,
    method = "fisher", n_permutations = 100, seed = 1)
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  expect_no_error(
    plot_critical_edges(populations, result,
                        communities = rep(1:3, each = 4)))
})

test_that("plot_critical_edges accepts pre-computed central graphs and a layout matrix", {
  skip_on_cran()
  skip_if_not_installed("igraph")
  populations <- make_populations(2)
  result <- identify_critical_links(populations, alpha = 0.05,
    method = "fisher", n_permutations = 100, seed = 2)
  centrals <- lapply(populations, compute_central_graph)
  lay <- igraph::layout_in_circle(
    igraph::graph_from_adjacency_matrix(centrals[[1]] > 0.5,
                                        mode = "undirected"))
  pdf(NULL); on.exit(dev.off(), add = TRUE)
  expect_no_error(
    plot_critical_edges(populations, result,
                        central_graphs = centrals,
                        layout = lay,
                        reference = "Patient",
                        vertex_label = FALSE))
})

test_that("plot_critical_edges validates input", {
  expect_error(plot_critical_edges(list(), list(critical_edges = NULL)),
               "non-empty list")
  expect_error(
    plot_critical_edges(list(A = list()), list(not_critical = NULL)),
    "identify_critical_links")
  populations <- make_populations(3)
  fake_result <- list(critical_edges = NULL)
  expect_error(
    plot_critical_edges(populations, fake_result,
                        communities = 1:5),
    "communities")
  expect_error(
    plot_critical_edges(populations, fake_result, reference = 99L),
    "out of range")
  expect_error(
    plot_critical_edges(populations, fake_result, reference = "Nope"),
    "not found")
})

# R/generate_community_graph.R

#' Generate a Random Symmetric Adjacency Matrix with Community Structure
#'
#' This function generates a random symmetric adjacency matrix representing
#' a brain network with community structure. Nodes within the same community
#' have a higher probability of being connected compared to nodes from different
#' communities.
#'
#' @param n_nodes An integer specifying the total number of nodes (brain regions).
#'   Default is 100.
#' @param n_communities An integer specifying the number of communities. Default is 4.
#' @param community_sizes An integer vector specifying the sizes of each community.
#'   If NULL, communities are of equal size. Default is NULL.
#' @param intra_prob A numeric value between 0 and 1, or a numeric vector of
#'   length \code{n_communities}, specifying the probability of an edge existing
#'   between nodes within the same community. If a scalar, the same probability
#'   is used for all communities. Default is 0.8.
#' @param inter_prob A numeric value between 0 and 1 specifying the probability
#'   of an edge existing between nodes from different communities. Default is 0.2.
#' @param seed An optional integer for setting the random seed to ensure reproducibility.
#'   Default is NULL.
#'
#' @return A symmetric binary adjacency matrix with no self-loops, representing
#'   a brain network with community structure.
#' @export
#'
#' @importFrom stats rbinom
#' @examples
#' # Generate a brain network with community structure
#' G <- generate_community_graph(n_nodes = 100, n_communities = 4, intra_prob = 0.8, inter_prob = 0.2)
generate_community_graph <- function(n_nodes = 100, n_communities = 4, community_sizes = NULL,
                                     intra_prob = 0.8, inter_prob = 0.2, seed = NULL) {
  if (!is.null(seed)) {
    set.seed(seed)
  }

  if (!is.numeric(n_nodes) || length(n_nodes) != 1 || n_nodes <= 0) {
    stop("n_nodes must be a positive integer.")
  }
  n_nodes <- as.integer(n_nodes)
  
  if (!is.numeric(n_communities) || length(n_communities) != 1 || n_communities <= 0) {
    stop("n_communities must be a positive integer.")
  }
  n_communities <- as.integer(n_communities)
  
  if (is.null(community_sizes)) {
    # If community sizes are not provided, divide nodes equally
    base_size <- n_nodes %/% n_communities
    remainder <- n_nodes %% n_communities
    community_sizes <- rep(base_size, n_communities)
    if (remainder > 0) {
      community_sizes[1:remainder] <- community_sizes[1:remainder] + 1
    }
  } else {
    if (length(community_sizes) != n_communities) {
      stop("Length of community_sizes must equal n_communities.")
    }
    if (sum(community_sizes) != n_nodes) {
      stop("Sum of community_sizes must equal n_nodes.")
    }
  }

  if (!is.numeric(intra_prob) || any(intra_prob < 0) || any(intra_prob > 1)) {
    stop("intra_prob must be numeric value(s) between 0 and 1.")
  }
  if (length(intra_prob) == 1L) {
    intra_prob <- rep(intra_prob, n_communities)
  } else if (length(intra_prob) != n_communities) {
    stop("intra_prob must be a scalar or a vector of length n_communities.")
  }
  
  if (!is.numeric(inter_prob) || inter_prob < 0 || inter_prob > 1) {
    stop("inter_prob must be a numeric value between 0 and 1.")
  }

  # Initialize adjacency matrix
  G <- matrix(0, nrow = n_nodes, ncol = n_nodes)

  # Assign nodes to communities
  node_indices <- 1:n_nodes
  community_assignments <- rep(1:n_communities, times = community_sizes)

  # Create community blocks
  for (i in 1:n_communities) {
    nodes_in_i <- node_indices[community_assignments == i]
    # Edges within community i
    if (length(nodes_in_i) > 1) {
      G_intra <- rbinom(length(nodes_in_i) * (length(nodes_in_i) - 1) / 2, 1, intra_prob[i])
      G_intra_matrix <- matrix(0, nrow = length(nodes_in_i), ncol = length(nodes_in_i))
      G_intra_matrix[upper.tri(G_intra_matrix)] <- G_intra
      G_intra_matrix <- G_intra_matrix + t(G_intra_matrix)
      G[nodes_in_i, nodes_in_i] <- G_intra_matrix
    }
  }

  # Create inter-community edges
  if (n_communities >= 2L) {
    for (i in 1:(n_communities - 1)) {
      for (j in (i + 1):n_communities) {
        nodes_in_i <- node_indices[community_assignments == i]
        nodes_in_j <- node_indices[community_assignments == j]
        # Edges between community i and community j
        G_inter <- matrix(rbinom(length(nodes_in_i) * length(nodes_in_j), 1, inter_prob),
                          nrow = length(nodes_in_i), ncol = length(nodes_in_j))
        G[nodes_in_i, nodes_in_j] <- G_inter
        G[nodes_in_j, nodes_in_i] <- t(G_inter)
      }
    }
  }

  # Remove self-loops
  diag(G) <- 0

  return(G)
}

# tests/testthat/test-generate_community_graph.R

test_that("generate_community_graph returns a valid adjacency matrix", {
  set.seed(123)
  G <- generate_community_graph(n_nodes = 10, n_communities = 2,
                                intra_prob = 0.9, inter_prob = 0.1)

  expect_equal(dim(G), c(10, 10))
  expect_true(isSymmetric(G))
  expect_equal(diag(G), rep(0, 10))
  expect_true(all(G %in% c(0, 1)))

  # Intra-community edges should dominate
  comm <- rep(1:2, each = 5)
  intra <- sum(G[comm == 1, comm == 1]) + sum(G[comm == 2, comm == 2])
  inter <- sum(G[comm == 1, comm == 2])
  expect_true(intra > inter)
})

test_that("generate_community_graph is reproducible with seed", {
  G1 <- generate_community_graph(20, 2, intra_prob = 0.7, inter_prob = 0.2, seed = 99)
  G2 <- generate_community_graph(20, 2, intra_prob = 0.7, inter_prob = 0.2, seed = 99)
  expect_identical(G1, G2)
})

test_that("generate_community_graph respects custom community_sizes", {
  G <- generate_community_graph(n_nodes = 12, n_communities = 3,
                                community_sizes = c(2, 4, 6),
                                intra_prob = 0.8, inter_prob = 0.2, seed = 1)
  expect_equal(dim(G), c(12, 12))
  expect_true(isSymmetric(G))
})

test_that("generate_community_graph accepts a vector intra_prob", {
  set.seed(1)
  # Community 1 is dense, community 2 is sparse
  G <- generate_community_graph(n_nodes = 40, n_communities = 2,
                                intra_prob = c(0.95, 0.05),
                                inter_prob = 0.1)
  comm <- rep(1:2, each = 20)
  dens1 <- mean(G[comm == 1, comm == 1][upper.tri(G[comm == 1, comm == 1])])
  dens2 <- mean(G[comm == 2, comm == 2][upper.tri(G[comm == 2, comm == 2])])
  expect_true(dens1 > dens2)
})

test_that("generate_community_graph validates inputs", {
  expect_error(generate_community_graph(n_nodes = 0, n_communities = 2),
               "positive integer")
  expect_error(generate_community_graph(n_nodes = 10, n_communities = 0),
               "positive integer")
  expect_error(generate_community_graph(n_nodes = 10, n_communities = 2,
                                        community_sizes = c(3, 3)),
               "Sum of community_sizes")
  expect_error(generate_community_graph(n_nodes = 10, n_communities = 3,
                                        community_sizes = c(5, 5)),
               "Length of community_sizes")
  expect_error(generate_community_graph(n_nodes = 10, n_communities = 2,
                                        intra_prob = 1.5),
               "between 0 and 1")
  expect_error(generate_community_graph(n_nodes = 10, n_communities = 2,
                                        inter_prob = -0.1),
               "between 0 and 1")
  # Wrong-length intra_prob vector
  expect_error(generate_community_graph(n_nodes = 10, n_communities = 2,
                                        intra_prob = c(0.5, 0.5, 0.5)),
               "scalar or a vector")
})

test_that("generate_community_graph works with n_communities = 1", {
  # Regression: n_communities = 1 previously triggered `1:(n_communities-1)`
  # which is `1:0`, corrupting the inter-community loop.
  set.seed(1)
  G <- generate_community_graph(n_nodes = 10, n_communities = 1,
                                intra_prob = 0.7, inter_prob = 0.0)
  expect_equal(dim(G), c(10, 10))
  expect_true(isSymmetric(G))
  expect_equal(diag(G), rep(0, 10))
  expect_true(all(G %in% c(0, 1)))
})

# R/generate_random_graph.R

#' Generate a Random Symmetric Adjacency Matrix
#'
#' This function generates a random symmetric adjacency matrix representing
#' a brain network. The adjacency matrix is binary, with edges present based
#' on a specified probability.
#'
#' @param n_nodes An integer specifying the number of nodes (brain regions).
#' @param edge_prob A numeric value between 0 and 1 specifying the probability
#'   of an edge existing between any two nodes.
#'
#' @return A symmetric binary adjacency matrix with no self-loops.
#' @export
#'
#' @importFrom stats rbinom
#' @examples
#' G <- generate_random_graph(n_nodes = 10, edge_prob = 0.1)
generate_random_graph <- function(n_nodes, edge_prob = 0.1) {
  if (!is.numeric(n_nodes) || length(n_nodes) != 1 || n_nodes <= 0) {
    stop("n_nodes must be a positive integer.")
  }
  
  if (!is.numeric(edge_prob) || edge_prob < 0 || edge_prob > 1) {
    stop("edge_prob must be a numeric value between 0 and 1.")
  }
  
  # Initialize a matrix of zeros
  G <- matrix(0, nrow = n_nodes, ncol = n_nodes)
  
  # Get indices of upper triangle excluding the diagonal
  upper_indices <- which(upper.tri(G), arr.ind = TRUE)
  
  # Assign random edges to upper triangle
  G[upper_indices] <- rbinom(nrow(upper_indices), 1, edge_prob)
  
  # Mirror the upper triangle to the lower triangle to make it symmetric
  G <- G + t(G)
  
  # Ensure diagonal is zero (no self-loops)
  diag(G) <- 0
  
  return(G)
}

# tests/testthat/test-generate_random_graph.R

test_that("generate_random_graph returns a valid adjacency matrix", {
  set.seed(1)
  G <- generate_random_graph(n_nodes = 20, edge_prob = 0.3)

  expect_true(is.matrix(G))
  expect_equal(dim(G), c(20, 20))
  expect_true(isSymmetric(G))
  expect_equal(diag(G), rep(0, 20))
  expect_true(all(G %in% c(0, 1)))
})

test_that("generate_random_graph respects edge_prob boundaries", {
  # edge_prob = 0 -> empty graph
  G0 <- generate_random_graph(n_nodes = 10, edge_prob = 0)
  expect_equal(sum(G0), 0)

  # edge_prob = 1 -> complete graph (off-diagonal)
  G1 <- generate_random_graph(n_nodes = 10, edge_prob = 1)
  expect_equal(sum(G1), 10 * 9)   # all off-diagonal entries
  expect_equal(diag(G1), rep(0, 10))
})

test_that("generate_random_graph validates inputs", {
  expect_error(generate_random_graph(n_nodes = 0, edge_prob = 0.1),
               "positive integer")
  expect_error(generate_random_graph(n_nodes = -5, edge_prob = 0.1),
               "positive integer")
  expect_error(generate_random_graph(n_nodes = c(5, 10), edge_prob = 0.1),
               "positive integer")
  expect_error(generate_random_graph(n_nodes = 10, edge_prob = -0.1),
               "between 0 and 1")
  expect_error(generate_random_graph(n_nodes = 10, edge_prob = 1.1),
               "between 0 and 1")
  expect_error(generate_random_graph(n_nodes = 10, edge_prob = "a"),
               "numeric")
})

# R/get_critical_nodes.R

#' Extract Critical Nodes from Critical Edge Results
#'
#' Given the output of \code{\link{identify_critical_links}}, this function
#' identifies the unique nodes involved in the critical edges and summarizes
#' their participation. Each node is reported with the number of critical edges
#' it participates in (its \emph{critical degree}). If node labels are supplied,
#' they are included in the output.
#'
#' @param result The list returned by \code{\link{identify_critical_links}}.
#'   Must contain a \code{critical_edges} component (a data frame with columns
#'   \code{node1} and \code{node2}, or \code{NULL}).
#' @param node_labels An optional character vector of length \eqn{p} (the number
#'   of nodes in the network), where \code{node_labels[i]} is the label for
#'   node \code{i}. If \code{NULL} (the default), no labels are included.
#'
#' @return A data frame with the following columns, ordered by decreasing
#'   \code{critical_degree}:
#'   \describe{
#'     \item{node}{Integer node index.}
#'     \item{label}{Character label for the node (only present when
#'       \code{node_labels} is not \code{NULL}).}
#'     \item{critical_degree}{Number of critical edges incident on this node.}
#'   }
#'   If no critical edges were found (\code{result$critical_edges} is
#'   \code{NULL} or has zero rows), an empty data frame with the same columns
#'   is returned.
#'
#' @export
#'
#' @examples
#' # Generate two synthetic populations with different community structure
#' control <- generate_category_graphs(n_graphs = 15, n_nodes = 10,
#'   n_communities = 2, base_intra_prob = 0.8, base_inter_prob = 0.2, seed = 1)
#' patient <- generate_category_graphs(n_graphs = 15, n_nodes = 10,
#'   n_communities = 2, base_intra_prob = 0.5, base_inter_prob = 0.5, seed = 2)
#' populations <- list(Control = control, Patient = patient)
#'
#' # Identify critical edges
#' result <- identify_critical_links(populations, alpha = 0.05,
#'   method = "fisher", n_permutations = 200, seed = 42)
#'
#' # Extract critical nodes (no labels)
#' get_critical_nodes(result)
#'
#' # With labels
#' labels <- paste0("Region_", 1:10)
#' get_critical_nodes(result, node_labels = labels)
get_critical_nodes <- function(result, node_labels = NULL) {

  ## --- Input validation ---
  if (!is.list(result) || !"critical_edges" %in% names(result))
    stop("`result` must be the list returned by identify_critical_links().")

  if (!is.null(node_labels) && !is.character(node_labels))
    stop("`node_labels` must be a character vector or NULL.")

  ## --- Handle empty results ---
  ce <- result$critical_edges
  if (is.null(ce) || nrow(ce) == 0L) {
    out <- data.frame(node = integer(0), critical_degree = integer(0))
    if (!is.null(node_labels))
      out$label <- character(0)
    return(out)
  }

  ## --- Validate node_labels length against the data ---
  max_node <- max(ce$node1, ce$node2)
  if (!is.null(node_labels) && length(node_labels) < max_node)
    stop(sprintf(
      "`node_labels` has length %d but critical edges reference node %d.",
      length(node_labels), max_node))

  ## --- Compute critical degree per node ---
  all_nodes <- c(ce$node1, ce$node2)
  freq <- table(all_nodes)
  node_ids <- as.integer(names(freq))
  degrees  <- as.integer(freq)

  out <- data.frame(node = node_ids, critical_degree = degrees)

  if (!is.null(node_labels))
    out$label <- node_labels[out$node]

  ## --- Sort by critical_degree descending ---
  out <- out[order(out$critical_degree, decreasing = TRUE), ]
  rownames(out) <- NULL

  ## --- Reorder columns so label comes before degree if present ---
  if (!is.null(node_labels))
    out <- out[, c("node", "label", "critical_degree")]

  out
}

# R/identify_critical_links.R
# Fast, mathematically exact implementation.

#' Identify Critical Edges That Explain Population Differences
#'
#' Iteratively identifies the edges that drive the observed difference between
#' two or more populations of brain networks, using the permutation-based
#' L1-distance ANOVA test of Fraiman and Fraiman (2018). Edges are ranked by
#' their marginal p-value, then removed in batches until the global test is no
#' longer significant. The permutation null is re-evaluated incrementally via
#' a prefix-sum decomposition of T, which avoids recomputing the statistic
#' from scratch at every step.
#'
#' @param populations A named \code{list} with at least two elements. Each
#'   element is itself a list of square, binary, symmetric adjacency matrices
#'   (zero diagonal, no self-loops). All graphs must share the same number of
#'   nodes.
#' @param alpha Numeric significance level used both for the global test and
#'   for the iterative edge-removal stopping rule. Default \code{0.05}.
#' @param method Character string selecting the marginal edge test used to
#'   rank candidate edges. One of \code{"fisher"} (default),
#'   \code{"chi.squared"}, or \code{"prop"}. See
#'   \code{\link{compute_edge_pvalues}}.
#' @param adjust_method Character string passed to \code{\link[stats]{p.adjust}}
#'   for multiplicity correction of the marginal p-values. Default
#'   \code{"none"}.
#' @param batch_size Positive integer giving the number of edges to drop per
#'   iteration. Larger values speed up the procedure at the cost of coarser
#'   resolution. Default \code{1}.
#' @param n_permutations Positive integer giving the number of permutation
#'   replicates used to approximate the null distribution of T. Default
#'   \code{1000}.
#' @param a Positive numeric normalisation constant for the test statistic T
#'   (see \code{\link{compute_test_statistic}}). Default \code{1}.
#' @param seed Optional integer seed for reproducibility of the permutation
#'   replicates. Default \code{NULL}.
#'
#' @return A \code{list} with three components: \code{critical_edges} (a
#'   \code{data.frame} with columns \code{node1}, \code{node2}, \code{p_value},
#'   listing the edges removed until the test became non-significant, ordered
#'   from most to least significant; \code{NULL} if the global test was not
#'   significant), \code{edges_removed} (a list of length-2 integer vectors
#'   giving the \code{(i, j)} indices of the removed edges, in removal order),
#'   and \code{modified_populations} (the input \code{populations} with the
#'   critical edges zeroed out in every graph).
#'
#' @details The implementation exploits the fact that T decomposes as a sum of
#'   per-edge contributions \eqn{\Delta_e}. Removing an edge e is equivalent
#'   to subtracting \eqn{\Delta_e} from T, so prefix sums of the sorted
#'   \eqn{\Delta_e} values give the test statistic after any number of
#'   removals in O(1). The permutation null is built once via a single BLAS
#'   matrix multiplication and reused for every iteration, reducing the
#'   overall complexity from O(K * B * |E| * m) (naive) to O(B * |E| * m).
#'
#'   Permutation replicates are generated by uniformly permuting the group
#'   labels across the pooled sample of all graphs, while preserving the
#'   original group sizes (sampling without replacement).
#'
#' @references
#' Fraiman, D. and Fraiman, R. (2018) An ANOVA approach for statistical
#' comparisons of brain networks. \emph{Scientific Reports}, 8, 4746.
#' \doi{10.1038/s41598-018-21688-0}.
#'
#' @seealso \code{\link{compute_test_statistic}},
#'   \code{\link{compute_edge_pvalues}}, \code{\link{get_critical_nodes}}.
#'
#' @export
#' @examples
#' \donttest{
#' set.seed(1)
#' control <- generate_category_graphs(
#'   n_graphs = 15, n_nodes = 10, n_communities = 2,
#'   base_intra_prob = 0.8, base_inter_prob = 0.2, seed = 1)
#' patient <- generate_category_graphs(
#'   n_graphs = 15, n_nodes = 10, n_communities = 2,
#'   base_intra_prob = 0.5, base_inter_prob = 0.5, seed = 2)
#' populations <- list(Control = control, Patient = patient)
#' result <- identify_critical_links(
#'   populations, alpha = 0.05, method = "fisher",
#'   n_permutations = 200, seed = 42)
#' head(result$critical_edges)
#' }
identify_critical_links <- function(populations,
                                    alpha          = 0.05,
                                    method         = "fisher",
                                    adjust_method  = "none",
                                    batch_size     = 1,
                                    n_permutations = 1000,
                                    a              = 1,
                                    seed           = NULL) {

  if (!is.null(seed)) set.seed(seed)
  if (!is.list(populations) || length(populations) < 2)
    stop("`populations` must be a list with at least 2 groups.")
  if (batch_size  < 1L) stop("`batch_size` must be >= 1.")
  if (n_permutations < 1L) stop("`n_permutations` must be >= 1.")

  Npop <- sapply(populations, length)
  m    <- length(populations)
  n    <- sum(Npop)

  make_key <- function(i, j) paste(i, j, sep = "-")

  freq       <- compute_edge_frequencies(populations)
  edge_pvals <- compute_edge_pvalues(freq$edge_counts, Npop,
                                     method        = method,
                                     adjust_method = adjust_method)
  edge_df    <- rank_edges(edge_pvals)
  n_edges    <- nrow(edge_df)
  if (n_edges == 0L)
    return(list(critical_edges       = NULL,
                edges_removed        = list(),
                modified_populations = populations))

  .edge_deltas <- function(edge_counts) {
    n_nodes <- dim(edge_counts)[1]
    idx     <- which(upper.tri(matrix(0, n_nodes, n_nodes)), arr.ind = TRUE)
    if (nrow(idx) == 0L)
      return(list(indices = idx, deltas = numeric(0)))

    counts <- do.call(cbind, lapply(seq_len(m),
                     function(k) edge_counts[ , , k][idx]))
    if (!is.matrix(counts))
      counts <- matrix(counts, nrow = 1L)

    p_mat <- sweep(counts, 2, Npop, "/")
    p_tot <- rowSums(counts) / n
    Ptot  <- matrix(p_tot, nrow = nrow(counts), ncol = m)

    d_mat <- 2 * p_mat * (1 - p_mat)
    D_mat <- p_mat + Ptot - 2 * p_mat * Ptot

    coef_d <- sqrt(Npop) * (Npop / (Npop - 1))
    coef_D <- sqrt(Npop) * (n    / (n    - 1))

    delta  <- (sqrt(m) / a) *
              (d_mat %*% coef_d - D_mat %*% coef_D)[, 1]

    list(indices = idx, deltas = delta)
  }

  .edge_deltas_matrix <- function(counts_list) {
    E   <- nrow(counts_list[[1]])
    B   <- ncol(counts_list[[1]])

    p_list  <- lapply(seq_len(m), function(k) counts_list[[k]] / Npop[k])
    C_total <- Reduce("+", counts_list)
    p_tot   <- C_total / n

    coef_d <- sqrt(Npop) * (Npop / (Npop - 1))
    coef_D <- sqrt(Npop) * (n    / (n    - 1))

    delta_mat <- matrix(0, nrow = E, ncol = B)
    for (k in seq_len(m)) {
      pk  <- p_list[[k]]
      d_k <- 2 * pk * (1 - pk)
      D_k <- pk + p_tot - 2 * pk * p_tot
      delta_mat <- delta_mat + coef_d[k] * d_k - coef_D[k] * D_k
    }
    (sqrt(m) / a) * delta_mat
  }

  obs_delta_info <- .edge_deltas(freq$edge_counts)

  key_all   <- make_key(obs_delta_info$indices[, 1], obs_delta_info$indices[, 2])
  map_idx   <- match(make_key(edge_df$node1, edge_df$node2), key_all)
  delta_ord <- obs_delta_info$deltas[map_idx]

  prefix_obs <- cumsum(delta_ord)
  T0         <- sum(obs_delta_info$deltas)

  all_graphs <- unlist(populations, recursive = FALSE)
  ut_idx     <- obs_delta_info$indices
  X <- do.call(rbind, lapply(all_graphs, function(A) A[ut_idx]))
  storage.mode(X) <- "double"

  perm_mat <- replicate(n_permutations, sample.int(n))
  cumNpop  <- c(0L, cumsum(Npop))
  W_list <- lapply(seq_len(m), function(k) {
    rows    <- (cumNpop[k] + 1L):cumNpop[k + 1L]
    idx_all <- as.vector(perm_mat[rows, , drop = FALSE])
    rep_id  <- rep(seq_len(n_permutations), each = Npop[k])
    lin_idx <- (rep_id - 1L) * n + idx_all
    W <- matrix(tabulate(lin_idx, nbins = n * n_permutations),
                nrow = n, ncol = n_permutations)
    storage.mode(W) <- "double"
    W
  })

  C_list      <- lapply(W_list, function(W) crossprod(X, W))
  delta_all   <- .edge_deltas_matrix(C_list)
  perm_deltas <- t(delta_all[map_idx, , drop = FALSE])

  T_perm0     <- rowSums(perm_deltas)
  prefix_perm <- t(apply(perm_deltas, 1, cumsum))

  initial_p <- mean(T_perm0 < T0)
  if (initial_p > alpha) {
    warning("Initial test is not significant (p = ",
            round(initial_p, 4),
            "). Populations may be identical or differences too small to detect.")
    return(list(critical_edges       = NULL,
                edges_removed        = list(),
                modified_populations = populations))
  }

  batch_steps <- seq(batch_size, n_edges, by = batch_size)
  if (length(batch_steps) == 0L || batch_steps[length(batch_steps)] != n_edges)
    batch_steps <- c(batch_steps, n_edges)

  delta_vec  <- T_perm0 - T0
  R_at_steps <- sweep(prefix_perm[, batch_steps, drop = FALSE],
                      2, prefix_obs[batch_steps])
  p_at_steps <- colMeans(R_at_steps > delta_vec)

  nonsig_idx <- which(p_at_steps > alpha)[1]
  k_removed  <- if (is.na(nonsig_idx)) n_edges else batch_steps[nonsig_idx]

  edges_removed <- vector("list", k_removed)
  for (idx in seq_len(k_removed)) {
    i <- edge_df$node1[idx]; j <- edge_df$node2[idx]
    for (pop in seq_along(populations))
      for (g in seq_len(Npop[pop])) {
        populations[[pop]][[g]][i, j] <- 0
        populations[[pop]][[g]][j, i] <- 0
      }
    edges_removed[[idx]] <- c(i, j)
  }

  critical_edges <- if (k_removed > 0) edge_df[seq_len(k_removed), ] else NULL

  list(critical_edges       = critical_edges,
       edges_removed        = edges_removed,
       modified_populations = populations)
}

# tests/testthat/test-identify_critical_links.R

test_that("identify_critical_links works correctly", {
  skip_on_cran()
  
  # Generate synthetic populations
  set.seed(123)
  control_graphs <- generate_category_graphs(n_graphs = 10, n_nodes = 20, n_communities = 2,
                                             base_intra_prob = 0.8, base_inter_prob = 0.2, seed = 1)
  patient_graphs <- generate_category_graphs(n_graphs = 10, n_nodes = 20, n_communities = 2,
                                             base_intra_prob = 0.6, base_inter_prob = 0.4, seed = 2)
  populations <- list(Control = control_graphs, Patient = patient_graphs)
  
  # Run the function
  result <- identify_critical_links(populations, alpha = 0.05, method = "fisher", adjust_method = "none")
  
  # Check that critical edges are identified
  expect_true(!is.null(result$critical_edges))
  expect_true(length(result$edges_removed) > 0)
  
  # Check that modified populations are returned
  expect_equal(length(result$modified_populations), 2)
  
  # Further checks can be added based on expected outcomes
})


# ---------------------------------------------------------------------------
# Helper: reference while-loop implementation for equivalence testing.
# This is the OLD step-5 logic extracted verbatim.  It takes the pre-computed
# prefix sums, T0, T_perm0, etc. and returns k_removed using the sequential
# while-loop that was replaced by the one-shot vectorised computation.
# ---------------------------------------------------------------------------
.while_loop_reference <- function(T0, T_perm0, prefix_obs, prefix_perm,
                                  n_edges, batch_size, alpha) {
  k_removed <- 0L
  continue  <- TRUE
  while (continue && k_removed < n_edges) {
    batch_end <- min(k_removed + batch_size, n_edges)
    T_obs_k   <- T0 - prefix_obs[batch_end]
    T_perm_k  <- T_perm0 - prefix_perm[, batch_end]
    p_k        <- mean(T_perm_k < T_obs_k)
    k_removed  <- batch_end
    continue   <- (p_k <= alpha)
  }
  k_removed
}

# ---------------------------------------------------------------------------
# Helper: one-shot implementation (mirrors new step-5 logic) operating on the
# same pre-computed data.
# ---------------------------------------------------------------------------
.one_shot_reference <- function(T0, T_perm0, prefix_obs, prefix_perm,
                                n_edges, batch_size, alpha) {
  batch_steps <- seq(batch_size, n_edges, by = batch_size)
  if (length(batch_steps) == 0L || batch_steps[length(batch_steps)] != n_edges)
    batch_steps <- c(batch_steps, n_edges)

  delta_vec  <- T_perm0 - T0
  R_at_steps <- sweep(prefix_perm[, batch_steps, drop = FALSE],
                      2, prefix_obs[batch_steps])
  p_at_steps <- colMeans(R_at_steps > delta_vec)

  nonsig_idx <- which(p_at_steps > alpha)[1]
  if (is.na(nonsig_idx)) n_edges else batch_steps[nonsig_idx]
}


test_that("one-shot p-values are exactly equal to while-loop (batch_size = 1)", {
  skip_on_cran()
  set.seed(42)

  # Build deterministic prefix-sum data from a known setup
  control <- generate_category_graphs(n_graphs = 15, n_nodes = 10,
    n_communities = 2, base_intra_prob = 0.8, base_inter_prob = 0.2, seed = 10)
  patient <- generate_category_graphs(n_graphs = 15, n_nodes = 10,
    n_communities = 2, base_intra_prob = 0.5, base_inter_prob = 0.5, seed = 20)
  pops <- list(Control = control, Patient = patient)

  # Run the full optimised function to extract internals.
  # We re-derive prefix sums the same way identify_critical_links does.
  Npop <- sapply(pops, length)
  m    <- length(pops); n_total <- sum(Npop)
  freq <- compute_edge_frequencies(pops)
  ep   <- compute_edge_pvalues(freq$edge_counts, Npop, method = "fisher")
  edf  <- rank_edges(ep)
  n_edges <- nrow(edf)

  # Observed deltas (re-use the package's internal helper via the full function)
  # Instead, compute manually to avoid relying on unexported internals:
  n_nodes <- dim(freq$edge_counts)[1]
  ut      <- which(upper.tri(matrix(0, n_nodes, n_nodes)), arr.ind = TRUE)
  counts  <- do.call(cbind, lapply(seq_len(m), function(k) freq$edge_counts[,,k][ut]))
  if (!is.matrix(counts)) counts <- matrix(counts, nrow = 1L)
  p_mat   <- sweep(counts, 2, Npop, "/")
  p_tot   <- rowSums(counts) / n_total
  Ptot    <- matrix(p_tot, nrow = nrow(counts), ncol = m)
  d_mat   <- 2 * p_mat * (1 - p_mat)
  D_mat   <- p_mat + Ptot - 2 * p_mat * Ptot
  coef_d  <- sqrt(Npop) * (Npop / (Npop - 1))
  coef_D  <- sqrt(Npop) * (n_total / (n_total - 1))
  obs_deltas <- (sqrt(m) / 1) * (d_mat %*% coef_d - D_mat %*% coef_D)[, 1]

  make_key <- function(i, j) paste(i, j, sep = "-")
  key_all  <- make_key(ut[, 1], ut[, 2])
  map_idx  <- match(make_key(edf$node1, edf$node2), key_all)
  delta_ord <- obs_deltas[map_idx]
  prefix_obs <- cumsum(delta_ord)
  T0 <- sum(obs_deltas)

  # Permutation (deterministic via set.seed)
  all_graphs <- unlist(pops, recursive = FALSE)
  B <- 500
  set.seed(999)
  X <- do.call(rbind, lapply(all_graphs, function(A) A[ut]))
  storage.mode(X) <- "double"
  perm_mat <- replicate(B, sample.int(n_total))
  cumNpop  <- c(0L, cumsum(Npop))
  W_list <- lapply(seq_len(m), function(k) {
    rows    <- (cumNpop[k] + 1L):cumNpop[k + 1L]
    idx_all <- as.vector(perm_mat[rows, , drop = FALSE])
    rep_id  <- rep(seq_len(B), each = Npop[k])
    lin_idx <- (rep_id - 1L) * n_total + idx_all
    W <- matrix(tabulate(lin_idx, nbins = n_total * B), nrow = n_total, ncol = B)
    storage.mode(W) <- "double"; W
  })
  C_list <- lapply(W_list, function(W) crossprod(X, W))
  # Compute permutation deltas
  p_list  <- lapply(seq_len(m), function(k) C_list[[k]] / Npop[k])
  C_total <- Reduce("+", C_list); p_tot_b <- C_total / n_total
  delta_mat <- matrix(0, nrow = nrow(ut), ncol = B)
  for (k in seq_len(m)) {
    pk <- p_list[[k]]
    d_k <- 2 * pk * (1 - pk); D_k <- pk + p_tot_b - 2 * pk * p_tot_b
    delta_mat <- delta_mat + coef_d[k] * d_k - coef_D[k] * D_k
  }
  delta_mat <- (sqrt(m) / 1) * delta_mat
  perm_deltas <- t(delta_mat[map_idx, , drop = FALSE])
  T_perm0     <- rowSums(perm_deltas)
  prefix_perm <- t(apply(perm_deltas, 1, cumsum))

  K_loop    <- .while_loop_reference(T0, T_perm0, prefix_obs, prefix_perm,
                                     n_edges, batch_size = 1, alpha = 0.05)
  K_oneshot <- .one_shot_reference(T0, T_perm0, prefix_obs, prefix_perm,
                                   n_edges, batch_size = 1, alpha = 0.05)

  expect_identical(K_oneshot, K_loop)
})


test_that("one-shot p-values are exactly equal to while-loop (batch_size > 1)", {
  skip_on_cran()
  set.seed(42)

  control <- generate_category_graphs(n_graphs = 15, n_nodes = 10,
    n_communities = 2, base_intra_prob = 0.8, base_inter_prob = 0.2, seed = 10)
  patient <- generate_category_graphs(n_graphs = 15, n_nodes = 10,
    n_communities = 2, base_intra_prob = 0.5, base_inter_prob = 0.5, seed = 20)
  pops <- list(Control = control, Patient = patient)

  # Run the actual function with different batch sizes; same seed -> same
  # permutation draws -> identical result apart from batch-size stepping.
  for (bs in c(2, 5, 10)) {
    r1 <- identify_critical_links(pops, alpha = 0.05, method = "fisher",
                                  batch_size = bs, n_permutations = 500, seed = 77)
    r2 <- identify_critical_links(pops, alpha = 0.05, method = "fisher",
                                  batch_size = bs, n_permutations = 500, seed = 77)
    # Same seed -> must be identical
    expect_identical(r1$critical_edges, r2$critical_edges,
                     info = paste("batch_size =", bs))
    expect_identical(r1$edges_removed, r2$edges_removed,
                     info = paste("batch_size =", bs))
  }
})


test_that("identify_critical_links returns NULL for identical populations", {
  skip_on_cran()

  identical_graphs <- generate_category_graphs(n_graphs = 20, n_nodes = 10,
    n_communities = 2, base_intra_prob = 0.5, base_inter_prob = 0.5, seed = 1)
  # Both populations share the same generating parameters
  pops <- list(A = identical_graphs[1:10], B = identical_graphs[11:20])

  expect_warning(
    result <- identify_critical_links(pops, alpha = 0.05, method = "fisher",
                                      n_permutations = 500, seed = 42),
    "not significant"
  )
  expect_null(result$critical_edges)
  expect_equal(length(result$edges_removed), 0)
})


test_that("identify_critical_links handles very different populations", {
  skip_on_cran()

  # Maximally different: one population is all-zero, the other is dense
  p <- 8
  zeros <- replicate(10, matrix(0L, p, p), simplify = FALSE)
  dense <- replicate(10, {
    G <- generate_random_graph(p, edge_prob = 0.9); G
  }, simplify = FALSE)
  pops <- list(Empty = zeros, Dense = dense)

  result <- identify_critical_links(pops, alpha = 0.05, method = "fisher",
                                    n_permutations = 500, seed = 42)
  # Should identify many critical edges

  expect_true(!is.null(result$critical_edges))
  expect_true(nrow(result$critical_edges) > 0)
})


test_that("identify_critical_links with batch_size = n_edges", {
  skip_on_cran()

  control <- generate_category_graphs(n_graphs = 10, n_nodes = 8,
    n_communities = 2, base_intra_prob = 0.8, base_inter_prob = 0.2, seed = 1)
  patient <- generate_category_graphs(n_graphs = 10, n_nodes = 8,
    n_communities = 2, base_intra_prob = 0.4, base_inter_prob = 0.6, seed = 2)
  pops <- list(Control = control, Patient = patient)

  n_edges <- 8 * 7 / 2  # 28

  result <- identify_critical_links(pops, alpha = 0.05, method = "fisher",
                                    batch_size = n_edges, n_permutations = 500,
                                    seed = 42)
  # batch_size = n_edges means only one p-value is evaluated (at step n_edges).
  # If that p-value is still <= alpha, all edges are removed.
  # If not, all edges are still reported as removed (the first batch covers everything).
  expect_true(is.null(result$critical_edges) || nrow(result$critical_edges) > 0)
})


# ---------------------------------------------------------------------------
# Input validation and edge cases
# ---------------------------------------------------------------------------
test_that("identify_critical_links validates inputs", {
  G <- matrix(0L, 4, 4)
  pops_ok <- list(A = list(G, G, G), B = list(G, G, G))

  expect_error(identify_critical_links("not a list"), "at least 2 groups")
  expect_error(identify_critical_links(list(A = list(G, G))), "at least 2 groups")
  expect_error(identify_critical_links(pops_ok, batch_size = 0),
               "batch_size")
  expect_error(identify_critical_links(pops_ok, n_permutations = 0),
               "n_permutations")
})

test_that("identify_critical_links is deterministic given a seed", {
  skip_on_cran()
  set.seed(1)
  ctrl <- generate_category_graphs(n_graphs = 10, n_nodes = 8,
    n_communities = 2, base_intra_prob = 0.8, base_inter_prob = 0.2, seed = 1)
  dis  <- generate_category_graphs(n_graphs = 10, n_nodes = 8,
    n_communities = 2, base_intra_prob = 0.4, base_inter_prob = 0.6, seed = 2)
  pops <- list(A = ctrl, B = dis)

  r1 <- identify_critical_links(pops, n_permutations = 200, seed = 5)
  r2 <- identify_critical_links(pops, n_permutations = 200, seed = 5)
  expect_identical(r1, r2)
  expect_identical(r1, r2)
})

test_that("identify_critical_links zeroes out returned modified_populations", {
  skip_on_cran()
  set.seed(1)
  ctrl <- generate_category_graphs(n_graphs = 10, n_nodes = 8,
    n_communities = 2, base_intra_prob = 0.9, base_inter_prob = 0.1, seed = 1)
  dis  <- generate_category_graphs(n_graphs = 10, n_nodes = 8,
    n_communities = 2, base_intra_prob = 0.1, base_inter_prob = 0.9, seed = 2)
  pops <- list(A = ctrl, B = dis)

  result <- identify_critical_links(pops, n_permutations = 200, seed = 11)
  if (!is.null(result$critical_edges) && nrow(result$critical_edges) > 0) {
    # Every critical edge must be zero in every graph of every population
    for (e in seq_len(nrow(result$critical_edges))) {
      i <- result$critical_edges$node1[e]; j <- result$critical_edges$node2[e]
      for (pop in result$modified_populations)
        for (G in pop) {
          expect_equal(G[i, j], 0)
          expect_equal(G[j, i], 0)
        }
    }
  }
})

test_that("identify_critical_links agrees with naive implementation on small data", {
  skip_on_cran()
  # The naive implementation is not exported; access it via :::
  naive <- BrainNetTest:::identify_critical_links_naive

  set.seed(123)
  ctrl <- generate_category_graphs(n_graphs = 8, n_nodes = 6,
    n_communities = 2, base_intra_prob = 0.9, base_inter_prob = 0.1, seed = 1)
  dis  <- generate_category_graphs(n_graphs = 8, n_nodes = 6,
    n_communities = 2, base_intra_prob = 0.1, base_inter_prob = 0.9, seed = 2)
  pops <- list(A = ctrl, B = dis)

  # Use the same seed inside both; Monte Carlo noise is small with B=500
  # but deltas are identical between implementations by construction, so
  # disagreement must stay within a small tolerance in k_removed.
  fast  <- identify_critical_links(pops, n_permutations = 500, seed = 321)
  slow  <- naive(pops, n_permutations = 500, seed = 321)

  k_fast <- if (is.null(fast$critical_edges)) 0 else nrow(fast$critical_edges)
  k_slow <- if (is.null(slow$critical_edges)) 0 else nrow(slow$critical_edges)

  # The fast and naive procedures draw permutations in different orders,
  # so k may differ by a small amount; require close agreement.
  expect_lte(abs(k_fast - k_slow), 3)
})

# R/compute_distance.R

#' Compute the Manhattan Norm Distance Between Two Graphs
#'
#' This function calculates the Manhattan norm (L1 norm) distance between two
#' adjacency matrices representing brain networks.
#'
#' @param G A square adjacency matrix representing a brain network.
#' @param M A square adjacency matrix representing the central brain network.
#'
#' @return A numeric value representing the Manhattan distance between G and M.
#' @export
#'
#' @importFrom stats rbinom
#' @examples
#' # Generate synthetic data
#' G1 <- generate_random_graph(n_nodes = 5, edge_prob = 0.1)
#' G2 <- generate_random_graph(n_nodes = 5, edge_prob = 0.1)
#' central_graph <- compute_central_graph(list(G1, G2))
#' distance <- compute_distance(G1, central_graph)
compute_distance <- function(G, M) {
  if (!is.matrix(G) || !is.matrix(M)) {
    stop("Both G and M must be matrices.")
  }
  
  if (!all(dim(G) == dim(M))) {
    stop("G and M must have the same dimensions.")
  }
  
  return(sum(abs(G - M)))
}

# R/compute_central_graph.R

#' Compute the Central (Representative) Graph for a Population
#'
#' This function computes the central graph for a given population by averaging
#' the adjacency matrices of all graphs within that population.
#'
#' @param graph_list A list of adjacency matrices representing brain networks.
#'
#' @return A single adjacency matrix representing the central graph of the population.
#' @export
#'
#' @examples
#' # Generate synthetic Control data
#' Control <- list(
#'   generate_random_graph(n_nodes = 5, edge_prob = 0.1),
#'   generate_random_graph(n_nodes = 5, edge_prob = 0.1)
#' )
#' central_graph <- compute_central_graph(Control)
compute_central_graph <- function(graph_list) {
  if (length(graph_list) == 0) {
    stop("graph_list is empty.")
  }
  
  # Check if all matrices are square and have the same dimensions
  n_nodes <- nrow(graph_list[[1]])
  for (G in graph_list) {
    if (!is.matrix(G)) {
      stop("All elements of graph_list must be matrices.")
    }
    if (nrow(G) != n_nodes || ncol(G) != n_nodes) {
      stop("All adjacency matrices must be square and of the same dimensions.")
    }
  }
  
  # Initialize a matrix of zeros
  central_matrix <- matrix(0, nrow = n_nodes, ncol = n_nodes)
  
  # Sum all adjacency matrices
  for (G in graph_list) {
    central_matrix <- central_matrix + G
  }
  
  # Average the summed matrix
  central_matrix <- central_matrix / length(graph_list)
  
  return(central_matrix)
}

# R/rank_edges.R

#' Rank Edges Based on P-Values
#'
#' This function ranks edges based on their p-values obtained from statistical tests,
#' ordering them from lowest to highest p-value (most to least significant).
#'
#' @param edge_pvalues A square matrix of p-values for each edge, typically obtained from \code{compute_edge_pvalues}.
#' @return A data frame with edges and their corresponding p-values, ordered from most significant.
#' @export
#'
#' @examples
#' # Generate synthetic populations
#' control_graphs <- generate_category_graphs(
#'   n_graphs = 5,
#'   n_nodes = 10,
#'   n_communities = 2,
#'   base_intra_prob = 0.8,
#'   base_inter_prob = 0.2,
#'   seed = 1
#' )
#' patient_graphs <- generate_category_graphs(
#'   n_graphs = 5,
#'   n_nodes = 10,
#'   n_communities = 2,
#'   base_intra_prob = 0.6,
#'   base_inter_prob = 0.4,
#'   seed = 2
#' )
#' populations <- list(Control = control_graphs, Patient = patient_graphs)
#' 
#' # Compute edge frequencies
#' frequencies <- compute_edge_frequencies(populations)
#' edge_counts <- frequencies$edge_counts
#' N <- sapply(populations, length)
#' 
#' # Compute p-values for edge differences
#' edge_pvalues <- compute_edge_pvalues(edge_counts, N)
#' 
#' # Rank edges based on p-values
#' edge_df <- rank_edges(edge_pvalues)
#' # View the top ranked edges
#' head(edge_df)
rank_edges <- function(edge_pvalues) {
  # Convert upper triangle of p-value matrix to a vector
  edge_indices <- which(upper.tri(edge_pvalues), arr.ind = TRUE)
  p_values <- edge_pvalues[edge_indices]
  
  # Create a data frame
  edge_df <- data.frame(
    node1 = edge_indices[, 1],
    node2 = edge_indices[, 2],
    p_value = p_values
  )
  
  # Order by p-value ascending (most significant first)
  edge_df <- edge_df[order(edge_df$p_value, decreasing = FALSE), ]
  
  return(edge_df)
}

# tests/testthat/test-rank_edges.R

test_that("rank_edges orders by ascending p-value", {
  pv <- matrix(1, 4, 4)
  pv[1, 2] <- pv[2, 1] <- 0.01
  pv[1, 3] <- pv[3, 1] <- 0.50
  pv[1, 4] <- pv[4, 1] <- 0.20
  pv[2, 3] <- pv[3, 2] <- 0.05
  pv[2, 4] <- pv[4, 2] <- 0.80
  pv[3, 4] <- pv[4, 3] <- 0.001

  edf <- rank_edges(pv)

  expect_s3_class(edf, "data.frame")
  expect_equal(names(edf), c("node1", "node2", "p_value"))
  # 4 nodes -> 6 upper-triangle edges
  expect_equal(nrow(edf), 6L)
  # Sorted ascending
  expect_true(all(diff(edf$p_value) >= 0))
  # Most significant first: (3,4) with 0.001
  expect_equal(edf$node1[1], 3L)
  expect_equal(edf$node2[1], 4L)
  expect_equal(edf$p_value[1], 0.001)
})

test_that("rank_edges only reports upper-triangle edges (i < j)", {
  set.seed(1)
  pv <- matrix(runif(25), 5, 5)
  pv <- (pv + t(pv)) / 2            # symmetrise
  diag(pv) <- 1
  edf <- rank_edges(pv)
  expect_true(all(edf$node1 < edf$node2))
  expect_equal(nrow(edf), 5 * 4 / 2)
})

test_that("rank_edges handles the 2-node degenerate case", {
  pv <- matrix(c(1, 0.02, 0.02, 1), 2, 2)
  edf <- rank_edges(pv)
  expect_equal(nrow(edf), 1L)
  expect_equal(edf$node1, 1L)
  expect_equal(edf$node2, 2L)
  expect_equal(edf$p_value, 0.02)
})

# R/compute_test_statistic.R

#' Compute the Test Statistic T for Brain Network Populations
#'
#' This function computes the test statistic T to assess whether different
#' populations of brain networks originate from the same distribution.
#'
#' @param populations A named list where each element is a list of adjacency
#'   matrices for a population. Example:
#'   \code{list(Control = list(G1, G2, ...), PatientA = list(G1, G2, ...), PatientB = list(G1, G2, ...))}
#' @param a A normalization constant. Default is 1.
#'
#' @return A numeric value representing the test statistic T.
#' @export
#'
#' @importFrom stats rbinom
#' @examples
#' # Generate synthetic populations data
#' Control <- list(
#'   generate_random_graph(n_nodes = 5, edge_prob = 0.1),
#'   generate_random_graph(n_nodes = 5, edge_prob = 0.1)
#' )
#' PatientA <- list(
#'   generate_random_graph(n_nodes = 5, edge_prob = 0.15),
#'   generate_random_graph(n_nodes = 5, edge_prob = 0.15)
#' )
#' PatientB <- list(
#'   generate_random_graph(n_nodes = 5, edge_prob = 0.2),
#'   generate_random_graph(n_nodes = 5, edge_prob = 0.2)
#' )
#' 
#' populations <- list(Control = Control, PatientA = PatientA, PatientB = PatientB)
#' 
#' # Compute the test statistic T
#' T_value <- compute_test_statistic(populations, a = 1)
#' print(T_value)
compute_test_statistic <- function(populations, a = 1) {
  if (!is.list(populations) || length(populations) == 0) {
    stop("populations must be a non-empty list.")
  }
  
  m <- length(populations)  # Number of populations
  n_i <- sapply(populations, length)  # Number of graphs in each population
  n <- sum(n_i)  # Total number of graphs
  
  if (any(n_i < 2)) {
    stop("Each population must have at least two graphs.")
  }
  
  # Compute central graphs for each population
  central_graphs <- lapply(populations, compute_central_graph)
  
  # Compute average distance of each population to its central graph
  avg_d_Gi_Mi <- numeric(m)
  names(avg_d_Gi_Mi) <- names(populations)
  
  for (i in seq_along(populations)) {
    distances <- sapply(populations[[i]], compute_distance, M = central_graphs[[i]])
    avg_d_Gi_Mi[i] <- mean(distances)
  }
  
  # Compute average distance of all graphs to each central graph
  avg_d_G_Mi <- numeric(m)
  names(avg_d_G_Mi) <- names(populations)
  
  for (i in seq_along(central_graphs)) {
    all_distances <- numeric(n)
    idx <- 1
    for (j in seq_along(populations)) {
      distances <- sapply(populations[[j]], compute_distance, M = central_graphs[[i]])
      all_distances[idx:(idx + length(distances) - 1)] <- distances
      idx <- idx + length(distances)
    }
    avg_d_G_Mi[i] <- mean(all_distances)
  }
  
  # Compute the sum part of the test statistic
  sum_term <- 0
  for (i in seq_along(populations)) {
    term1 <- (n_i[i] / (n_i[i] - 1)) * avg_d_Gi_Mi[i]
    term2 <- (n / (n - 1)) * avg_d_G_Mi[i]
    sum_term <- sum_term + sqrt(n_i[i]) * (term1 - term2)
  }
  
  # Compute the test statistic T
  T <- (sqrt(m) / a) * sum_term
  
  return(T)
}

# R/compute_edge_pvalues.R

#' Compute P-Values for Edge Differences Between Populations
#'
#' This function computes p-values for the differences in edge presence between populations
#' using statistical tests (e.g., Fisher's exact test, chi-squared test).
#'
#' @param edge_counts An array of edge counts from \code{compute_edge_frequencies}.
#' @param N A vector of sample sizes for each population.
#' @param method The statistical test method to use: "fisher", "chi.squared", or "prop". Default is "fisher".
#' @param adjust_method The method for p-value adjustment for multiple testing. Default is "none".
#' @importFrom stats fisher.test chisq.test prop.test p.adjust
#' @return A matrix of p-values for each edge.
#' @export
#'
#' @examples
#' # Generate synthetic populations
#' control_graphs <- generate_category_graphs(
#'   n_graphs = 5,
#'   n_nodes = 10,
#'   n_communities = 2,
#'   base_intra_prob = 0.8,
#'   base_inter_prob = 0.2,
#'   intra_prob_variation = 0.05,
#'   inter_prob_variation = 0.05,
#'   seed = 1
#' )
#' patient_graphs <- generate_category_graphs(
#'   n_graphs = 5,
#'   n_nodes = 10,
#'   n_communities = 2,
#'   base_intra_prob = 0.6,
#'   base_inter_prob = 0.4,
#'   intra_prob_variation = 0.05,
#'   inter_prob_variation = 0.05,
#'   seed = 2
#' )
#' populations <- list(Control = control_graphs, Patient = patient_graphs)
#'
#' # Compute edge frequencies
#' frequencies <- compute_edge_frequencies(populations)
#' edge_counts <- frequencies$edge_counts
#' N <- sapply(populations, length)
#'
#' # Compute p-values for edge differences
#' edge_pvalues <- compute_edge_pvalues(edge_counts, N)
#' # View p-values for the first few edges
#' print(edge_pvalues[1:5, 1:5])
compute_edge_pvalues <- function(edge_counts, N, method = "fisher", adjust_method = "none") {
  n_nodes <- dim(edge_counts)[1]
  num_populations <- dim(edge_counts)[3]
  edge_pvalues <- matrix(1, nrow = n_nodes, ncol = n_nodes)
  
  # For each pair of nodes (edge)
  for (i in 1:(n_nodes - 1)) {
    for (j in (i + 1):n_nodes) {
      # Build contingency table
      counts <- matrix(0, nrow = num_populations, ncol = 2)
      for (k in 1:num_populations) {
        counts[k, 1] <- edge_counts[i, j, k]                # Edge present
        counts[k, 2] <- N[k] - edge_counts[i, j, k]         # Edge absent
      }
      
      # Perform the chosen statistical test
      if (method == "fisher") {
        test_result <- fisher.test(counts)
        p_value <- test_result$p.value
      } else if (method == "chi.squared") {
        # Check if any row or column sum is zero
        row_sums <- rowSums(counts)
        col_sums <- colSums(counts)
        
        if (any(row_sums == 0) || any(col_sums == 0)) {
          # If zero marginals, fall back to Fisher's exact test
          test_result <- fisher.test(counts)
          p_value <- test_result$p.value
        } else {
          # Check if expected counts might be too small for chi-square approximation
          total <- sum(counts)
          has_small_expected <- FALSE
          
          # Calculate expected counts and check if any are < 5
          for (r in seq_len(nrow(counts))) {
            for (c in seq_len(ncol(counts))) {
              expected <- row_sums[r] * col_sums[c] / total
              if (expected < 5) {
                has_small_expected <- TRUE
                break
              }
            }
            if (has_small_expected) break
          }
          
          # Use standard chi-square test (simulation can fail with certain data patterns)
          test_result <- chisq.test(counts, correct = TRUE)
          p_value <- test_result$p.value
        }
      } else if (method == "prop") {
        # Use prop.test for two populations
        if (num_populations != 2) stop("prop.test is only implemented for two populations.")
        
        # Check for zero values that might cause issues
        x <- c(edge_counts[i, j, 1], edge_counts[i, j, 2])
        n <- c(N[1], N[2])
        
        # Handle edge cases
        if (all(x == 0) || all(x == n)) {
          p_value <- 1.0  # No difference when all are identical
        } else {
          test_result <- prop.test(x, n)
          p_value <- test_result$p.value
        }
      } else {
        stop("Invalid method specified.")
      }
      
      edge_pvalues[i, j] <- p_value
      edge_pvalues[j, i] <- p_value  # Symmetric matrix
    }
  }
  
  # Adjust p-values for multiple testing if needed
  if (adjust_method != "none") {
    # Adjust using only the upper triangle, then mirror to the lower triangle.
    # Note: assigning the same vector to both upper.tri() and lower.tri() would
    # pair elements incorrectly (R traverses each triangle in column-major
    # order over different index sets), producing an asymmetric matrix. We
    # therefore write the upper triangle first and then transpose.
    ut <- upper.tri(edge_pvalues)
    p_values_vec <- edge_pvalues[ut]
    adjusted_p_values <- p.adjust(p_values_vec, method = adjust_method)
    edge_pvalues[ut] <- adjusted_p_values
    edge_pvalues[lower.tri(edge_pvalues)] <- t(edge_pvalues)[lower.tri(edge_pvalues)]
  }
  
  return(edge_pvalues)
}

# R/compute_edge_frequencies.R

#' Compute Edge Frequencies in Populations
#'
#' This function computes the frequency of each edge (connection between nodes)
#' across all graphs within each population.
#'
#' @param populations A list where each element is a population containing a list of graphs (adjacency matrices).
#' @return A list containing edge counts and edge proportions for each population.
#' @export
#'
#' @examples
#' # Generate synthetic populations
#' control_graphs <- generate_category_graphs(
#'   n_graphs = 5,
#'   n_nodes = 10,
#'   n_communities = 2,
#'   base_intra_prob = 0.8,
#'   base_inter_prob = 0.2,
#'   intra_prob_variation = 0.05,
#'   inter_prob_variation = 0.05,
#'   seed = 1
#' )
#' patient_graphs <- generate_category_graphs(
#'   n_graphs = 5,
#'   n_nodes = 10,
#'   n_communities = 2,
#'   base_intra_prob = 0.6,
#'   base_inter_prob = 0.4,
#'   intra_prob_variation = 0.05,
#'   inter_prob_variation = 0.05,
#'   seed = 2
#' )
#' populations <- list(Control = control_graphs, Patient = patient_graphs)
#'
#' # Compute edge frequencies
#' frequencies <- compute_edge_frequencies(populations)
#' # View edge counts for the first population
#' edge_counts_control <- frequencies$edge_counts[,,1]
#' print(edge_counts_control)
compute_edge_frequencies <- function(populations) {
  num_populations <- length(populations)
  N <- sapply(populations, length)
  
  # Assuming all graphs have the same dimensions
  n_nodes <- nrow(populations[[1]][[1]])
  
  # Initialize arrays to store edge counts and frequencies
  edge_counts <- array(0, dim = c(n_nodes, n_nodes, num_populations))
  
  # For each population
  for (k in seq_along(populations)) {
    # Sum adjacency matrices across all graphs in the population
    edge_counts[,,k] <- Reduce("+", populations[[k]])
  }
  
  # Calculate edge presence proportions
  edge_proportions <- edge_counts
  for (k in seq_along(populations)) {
    edge_proportions[,,k] <- edge_counts[,,k] / N[k]
  }
  
  return(list(edge_counts = edge_counts, edge_proportions = edge_proportions))
}

# tests/testthat/test-compute_edge_frequencies.R

test_that("compute_edge_frequencies returns correct shape and values", {
  # Build two deterministic populations (2 pops x 2 graphs x 3 nodes)
  G_full  <- matrix(c(0, 1, 1,
                      1, 0, 1,
                      1, 1, 0), nrow = 3, byrow = TRUE)
  G_empty <- matrix(0, 3, 3)

  pops <- list(A = list(G_full, G_full),
               B = list(G_empty, G_full))

  freq <- compute_edge_frequencies(pops)

  # Array dims: n_nodes x n_nodes x num_populations
  expect_equal(dim(freq$edge_counts), c(3, 3, 2))
  expect_equal(dim(freq$edge_proportions), c(3, 3, 2))

  # Pop A: both graphs full -> count = 2 on off-diagonal, proportion = 1
  expect_equal(freq$edge_counts[, , 1], G_full * 2)
  expect_equal(freq$edge_proportions[, , 1], G_full)

  # Pop B: one empty + one full -> count = 1 on off-diagonal, proportion = 0.5
  expect_equal(freq$edge_counts[, , 2], G_full * 1)
  expect_equal(freq$edge_proportions[, , 2], G_full * 0.5)
})

test_that("compute_edge_frequencies handles unequal group sizes", {
  G <- matrix(c(0, 1, 1, 0), nrow = 2)
  pops <- list(A = list(G, G, G),    # n_A = 3
               B = list(G))          # n_B = 1
  freq <- compute_edge_frequencies(pops)
  expect_equal(freq$edge_counts[, , 1], G * 3)
  expect_equal(freq$edge_counts[, , 2], G * 1)
  expect_equal(freq$edge_proportions[, , 1], G)
  expect_equal(freq$edge_proportions[, , 2], G)
})

# R/plot_critical_edges.R

#' Visualize Central Graphs and Critical Edges
#'
#' Produces a multi-panel figure summarising the output of
#' \code{\link{identify_critical_links}}: one panel per population showing the
#' weighted central graph, plus a final panel that highlights the critical
#' edges on a chosen reference central graph. This is the canonical
#' visualization of the \pkg{BrainNetTest} workflow and replaces the manual
#' \pkg{igraph} plumbing required to build the same figure.
#'
#' @param populations A named \code{list} of populations of adjacency matrices,
#'   as accepted by \code{\link{identify_critical_links}}. Used to compute the
#'   per-population central graphs unless \code{central_graphs} is supplied.
#' @param critical_links The object returned by
#'   \code{\link{identify_critical_links}}. Its \code{critical_edges} component
#'   provides the edges to highlight in the final panel.
#' @param communities Optional integer (or factor) vector of length
#'   \code{n_nodes} giving the community membership of every node. Used to
#'   color vertices consistently across panels. Default \code{NULL} (all
#'   vertices share the default \pkg{igraph} color).
#' @param layout Optional layout for the graphs. Either an
#'   \code{n_nodes x 2} numeric matrix of vertex coordinates (recommended, so
#'   that all panels share the same node positions) or an \pkg{igraph} layout
#'   function such as \code{\link[igraph]{layout_with_fr}}. Default
#'   \code{\link[igraph]{layout_in_circle}}.
#' @param central_graphs Optional list of pre-computed central graphs (one per
#'   population). If \code{NULL}, they are computed via
#'   \code{\link{compute_central_graph}}.
#' @param reference Either an integer index or a name selecting which
#'   population's central graph is used as the background of the
#'   critical-edges panel. Default \code{1L} (the first population).
#' @param threshold Numeric in \code{[0, 1]}. The reference central graph is
#'   binarised by \code{> threshold} for the critical-edges panel. Default
#'   \code{0.5}.
#' @param edge_scale Numeric multiplier applied to the central-graph edge
#'   weights to set \code{edge.width} in the per-population panels. Default
#'   \code{6}.
#' @param critical_color,critical_width Color and line width used to draw
#'   critical edges. Defaults \code{"red"} and \code{3}.
#' @param background_color,background_width Color and line width used to draw
#'   non-critical edges in the critical panel. Defaults \code{"grey80"} and
#'   \code{1}.
#' @param vertex_size Vertex size passed to \code{\link[igraph]{plot.igraph}}.
#'   Default \code{12}.
#' @param vertex_label Logical. If \code{TRUE} (default) vertex indices are
#'   shown; otherwise vertex labels are suppressed.
#' @param panel_titles Optional character vector of length
#'   \code{length(populations) + 1} with custom panel titles. Default uses
#'   \code{paste(names(populations), "Central Graph")} followed by
#'   \code{"Critical edges"}.
#' @param mfrow Optional integer vector of length 2 giving the
#'   \code{c(nrow, ncol)} arrangement passed to \code{\link[graphics]{par}}.
#'   By default a near-square grid is chosen automatically (e.g.\ a
#'   \code{2 x 2} grid for two populations, matching the standard
#'   Control / Patient / Critical layout).
#' @param ... Additional arguments forwarded to
#'   \code{\link[igraph]{plot.igraph}}. Note that \code{edge.width},
#'   \code{vertex.color}, \code{vertex.size}, \code{vertex.label} and
#'   \code{layout} are managed by this function and should not be passed via
#'   \code{...}.
#'
#' @return Invisibly returns \code{NULL}; called for its side effect of
#'   producing the multi-panel plot.
#'
#' @details The first \code{length(populations)} panels show the weighted
#'   central graph of each population, with edge widths proportional to the
#'   central-graph weights. The final panel binarises the chosen reference
#'   central graph at \code{threshold} and overlays the critical edges
#'   returned by \code{identify_critical_links()} in
#'   \code{critical_color}. Critical edges that are absent from the binarised
#'   reference graph are added so that they remain visible.
#'
#'   The graphics state (\code{par(mfrow, mar, oma)}) is restored on exit.
#'
#' @seealso \code{\link{identify_critical_links}},
#'   \code{\link{compute_central_graph}},
#'   \code{\link{get_critical_nodes}}.
#'
#' @export
#' @importFrom graphics par
#' @examples
#' \donttest{
#' set.seed(123)
#' community_sizes <- c(4, 2, 3, 3, 5)
#' control <- generate_category_graphs(
#'   n_graphs = 50, n_nodes = 17, n_communities = 5,
#'   community_sizes = community_sizes,
#'   base_intra_prob = rep(0.70, 5), base_inter_prob = 0.05,
#'   intra_prob_variation = 0.02, inter_prob_variation = 0.01)
#' patient <- generate_category_graphs(
#'   n_graphs = 50, n_nodes = 17, n_communities = 5,
#'   community_sizes = community_sizes,
#'   base_intra_prob = c(0.40, 0.70, 0.70, 0.70, 0.70),
#'   base_inter_prob = 0.05,
#'   intra_prob_variation = 0.02, inter_prob_variation = 0.01)
#' populations <- list(Control = control, Patient = patient)
#' result <- identify_critical_links(populations, alpha = 0.05,
#'   method = "fisher", n_permutations = 200, seed = 1)
#' communities <- rep(seq_along(community_sizes), times = community_sizes)
#' plot_critical_edges(populations, result, communities = communities)
#' }
plot_critical_edges <- function(populations,
                                critical_links,
                                communities = NULL,
                                layout = NULL,
                                central_graphs = NULL,
                                reference = 1L,
                                threshold = 0.5,
                                edge_scale = 6,
                                critical_color = "red",
                                critical_width = 3,
                                background_color = "grey80",
                                background_width = 1,
                                vertex_size = 12,
                                vertex_label = TRUE,
                                panel_titles = NULL,
                                mfrow = NULL,
                                ...) {
  if (!requireNamespace("igraph", quietly = TRUE)) {
    stop("Package 'igraph' is required. ",
         "Install it with install.packages('igraph').")
  }
  if (!is.list(populations) || length(populations) < 1L) {
    stop("'populations' must be a non-empty list of populations.")
  }
  if (is.null(names(populations))) {
    names(populations) <- paste0("Group", seq_along(populations))
  }
  if (!is.list(critical_links) ||
      !("critical_edges" %in% names(critical_links))) {
    stop("'critical_links' must be the output of identify_critical_links().")
  }

  ## Central graphs ---------------------------------------------------------
  if (is.null(central_graphs)) {
    central_graphs <- lapply(populations, compute_central_graph)
  }
  if (length(central_graphs) != length(populations)) {
    stop("'central_graphs' must have the same length as 'populations'.")
  }
  n_nodes <- nrow(central_graphs[[1L]])
  if (any(vapply(central_graphs, nrow, integer(1L)) != n_nodes)) {
    stop("All central graphs must have the same number of nodes.")
  }

  ## Reference index --------------------------------------------------------
  if (is.character(reference)) {
    ref_idx <- match(reference, names(populations))
    if (is.na(ref_idx)) {
      stop("'reference' name not found in names(populations).")
    }
  } else {
    ref_idx <- as.integer(reference)
    if (length(ref_idx) != 1L || is.na(ref_idx) ||
        ref_idx < 1L || ref_idx > length(populations)) {
      stop("'reference' index out of range.")
    }
  }

  ## Communities ------------------------------------------------------------
  if (!is.null(communities) && length(communities) != n_nodes) {
    stop("'communities' must have length equal to the number of nodes.")
  }

  ## Layout -----------------------------------------------------------------
  if (is.null(layout)) {
    layout <- igraph::layout_in_circle
  }

  ## Titles -----------------------------------------------------------------
  K <- length(populations)
  if (is.null(panel_titles)) {
    panel_titles <- c(paste(names(populations), "Central Graph"),
                      "Critical edges")
  } else if (length(panel_titles) != K + 1L) {
    stop("'panel_titles' must have length equal to length(populations) + 1.")
  }

  ## Layout grid ------------------------------------------------------------
  if (is.null(mfrow)) {
    n_panels <- K + 1L
    nc <- ceiling(sqrt(n_panels))
    nr <- ceiling(n_panels / nc)
    mfrow <- c(nr, nc)
  }

  old_par <- graphics::par(mfrow = mfrow,
                           mar = c(1, 1, 2, 1),
                           oma = c(0, 0, 0, 0))
  on.exit(graphics::par(old_par), add = TRUE)

  set_vertex_attrs <- function(g) {
    if (!is.null(communities)) {
      igraph::V(g)$color <- communities
    }
    igraph::V(g)$size <- vertex_size
    if (!isTRUE(vertex_label)) {
      igraph::V(g)$label <- NA
    }
    g
  }

  ## Per-population weighted central-graph panels --------------------------
  for (k in seq_len(K)) {
    M <- central_graphs[[k]]
    g <- igraph::graph_from_adjacency_matrix(M, mode = "undirected",
                                             weighted = TRUE, diag = FALSE)
    g <- set_vertex_attrs(g)
    w <- igraph::E(g)$weight
    if (is.null(w)) w <- 1
    plot(g, layout = layout, main = panel_titles[k],
         edge.width = w * edge_scale, ...)
  }

  ## Critical-edges panel ---------------------------------------------------
  ref_central <- central_graphs[[ref_idx]]
  bin <- ref_central > threshold
  diag(bin) <- FALSE
  g <- igraph::graph_from_adjacency_matrix(bin, mode = "undirected",
                                           diag = FALSE)
  g <- set_vertex_attrs(g)
  igraph::E(g)$color <- background_color
  igraph::E(g)$width <- background_width

  ce <- critical_links$critical_edges
  if (!is.null(ce) && nrow(ce) > 0L) {
    el <- igraph::as_edgelist(g)
    for (i in seq_len(nrow(ce))) {
      a <- ce$node1[i]
      b <- ce$node2[i]
      idx <- which((el[, 1L] == a & el[, 2L] == b) |
                   (el[, 1L] == b & el[, 2L] == a))
      if (length(idx) > 0L) {
        igraph::E(g)[idx]$color <- critical_color
        igraph::E(g)[idx]$width <- critical_width
      } else {
        ## Critical edge missing from the thresholded reference -- add it
        ## so that it appears in the figure.
        g <- igraph::add_edges(
          g, c(a, b),
          attr = list(color = critical_color, width = critical_width))
      }
    }
  }

  plot(g, layout = layout, main = panel_titles[K + 1L], ...)

  invisible(NULL)
}

---
title: "Generating and Analyzing Brain Networks with Community Structures"
author: "Maximiliano Martino, Daniel Fraiman"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Generating and Analyzing Brain Networks with Community Structures}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment  = "#>"
)
```

## Overview

This vignette shows the full `BrainNetTest` workflow on synthetic populations
of brain networks with community (block) structure. We simulate two groups of
graphs with different intra- and inter-community connection probabilities,
run the global L1-distance ANOVA test, and identify the edges and nodes that
drive the observed difference.

```{r}
library(BrainNetTest)
```

## Generating a single community-structured graph

`generate_community_graph()` samples a symmetric binary adjacency matrix with
a block-stochastic structure: edges within a community are drawn with
probability `intra_prob`, edges between communities with `inter_prob`.

```{r}
set.seed(42)
G <- generate_community_graph(
  n_nodes       = 40,
  n_communities = 4,
  intra_prob    = 0.8,
  inter_prob    = 0.2)

dim(G)
mean(G[upper.tri(G)])
```

### Visualising the network

A single community-structured adjacency matrix is best inspected directly
with `igraph`. The package's own visualisation helper,
`plot_critical_edges()`, is designed to summarise the *result* of an analysis
(per-population central graphs plus the identified critical edges) and is
illustrated at the end of this vignette.

## Simulating populations of graphs

`generate_category_graphs()` produces a list of graphs that share a common
community structure but vary slightly in their edge probabilities across
replicates, modelling natural between-subject variability.

```{r}
control <- generate_category_graphs(
  n_graphs             = 20,
  n_nodes              = 20,
  n_communities        = 2,
  base_intra_prob      = 0.8,
  base_inter_prob      = 0.2,
  intra_prob_variation = 0.05,
  inter_prob_variation = 0.05,
  seed                 = 1)

patient <- generate_category_graphs(
  n_graphs             = 20,
  n_nodes              = 20,
  n_communities        = 2,
  base_intra_prob      = 0.6,
  base_inter_prob      = 0.4,
  intra_prob_variation = 0.05,
  inter_prob_variation = 0.05,
  seed                 = 2)

populations <- list(Control = control, Patient = patient)
lengths(populations)
```

## Global test statistic

```{r}
T_obs <- compute_test_statistic(populations, a = 1)
T_obs
```

## Identifying critical edges

`identify_critical_links()` performs the full pipeline: marginal edge
p-values, permutation null for `T`, and iterative edge removal. It returns
the edges whose removal eliminates the group-level difference.

```{r}
result <- identify_critical_links(
  populations,
  alpha          = 0.05,
  method         = "fisher",
  n_permutations = 500,
  seed           = 42)

nrow(result$critical_edges)
head(result$critical_edges)
```

### Critical nodes

`get_critical_nodes()` aggregates the critical edges at the node level,
reporting the *critical degree* (number of critical edges incident on each
node):

```{r}
get_critical_nodes(result)
```

## Visualising the result

`plot_critical_edges()` produces a multi-panel figure that summarises the
analysis: one panel per population showing the (weighted) central graph,
plus a final panel that highlights the critical edges on the chosen
reference central graph. Communities can be passed in to color the vertices
consistently across panels.

```{r, eval = requireNamespace("igraph", quietly = TRUE), fig.width=7, fig.height=7}
plot_critical_edges(
  populations,
  result,
  communities = rep(seq_len(2), each = 10),
  reference   = "Control")
```

## References

Fraiman, D. and Fraiman, R. (2018) An ANOVA approach for statistical
comparisons of brain networks. *Scientific Reports*, 8, 4746.
<https://doi.org/10.1038/s41598-018-21688-0>

---
title: "Introduction to BrainNetTest"
author: "Maximiliano Martino, Daniel Fraiman"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Introduction to BrainNetTest}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup_intro, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment  = "#>"
)
```

## Overview

`BrainNetTest` implements the non-parametric L1-distance ANOVA test of Fraiman
and Fraiman (2018) for comparing populations of brain networks represented as
graphs. The package provides:

* utilities to build a central (mean) graph for a population and measure
  Manhattan (L1) distances between adjacency matrices;
* the group test statistic `T` and its permutation null distribution;
* a fast procedure (`identify_critical_links`) that pinpoints the edges
  driving between-group differences; and
* helpers to simulate synthetic community-structured graphs and to visualise
  networks.

This vignette introduces the basic API on simple synthetic data. The companion
vignette, *Generating and Analyzing Brain Networks with Community Structures*,
shows the full pipeline on community-structured populations.

## Installation

```r
# Release version (once on CRAN)
install.packages("BrainNetTest")

# Development version from GitHub
# install.packages("remotes")
remotes::install_github("mmaximiliano/BrainNetTest")
```

## A minimal example

```{r}
library(BrainNetTest)
set.seed(1)
```

### Generating random graphs

`generate_random_graph()` returns a symmetric binary adjacency matrix with no
self-loops:

```{r}
G <- generate_random_graph(n_nodes = 10, edge_prob = 0.2)
isSymmetric(G)
all(diag(G) == 0)
```

### Central graph and Manhattan distance

Given a population (list of adjacency matrices), the *central graph* is the
entry-wise mean; `compute_distance()` returns the L1 (Manhattan) distance
between two graphs:

```{r}
population <- replicate(
  5, generate_random_graph(n_nodes = 10, edge_prob = 0.2),
  simplify = FALSE)

central <- compute_central_graph(population)
compute_distance(population[[1]], central)
```

### The group test statistic `T`

`compute_test_statistic()` aggregates within- and between-group Manhattan
distances into the statistic `T`. Larger values indicate stronger evidence
that the groups differ:

```{r}
control <- replicate(
  15, generate_random_graph(n_nodes = 10, edge_prob = 0.20),
  simplify = FALSE)
patient <- replicate(
  15, generate_random_graph(n_nodes = 10, edge_prob = 0.40),
  simplify = FALSE)

populations <- list(Control = control, Patient = patient)
compute_test_statistic(populations, a = 1)
```

### Global test and critical edges

`identify_critical_links()` combines the global permutation test with an
iterative edge-removal procedure that returns the edges driving the observed
difference:

```{r}
result <- identify_critical_links(
  populations,
  alpha          = 0.05,
  method         = "fisher",
  n_permutations = 200,
  seed           = 42)

head(result$critical_edges)
```

The `get_critical_nodes()` helper summarises this result at the node level:

```{r}
get_critical_nodes(result)
```

## References

Fraiman, D. and Fraiman, R. (2018) An ANOVA approach for statistical
comparisons of brain networks. *Scientific Reports*, 8, 4746.
<https://doi.org/10.1038/s41598-018-21688-0>

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compute_central_graph.R
\name{compute_central_graph}
\alias{compute_central_graph}
\title{Compute the Central (Representative) Graph for a Population}
\usage{
compute_central_graph(graph_list)
}
\arguments{
\item{graph_list}{A list of adjacency matrices representing brain networks.}
}
\value{
A single adjacency matrix representing the central graph of the population.
}
\description{
This function computes the central graph for a given population by averaging
the adjacency matrices of all graphs within that population.
}
\examples{
# Generate synthetic Control data
Control <- list(
  generate_random_graph(n_nodes = 5, edge_prob = 0.1),
  generate_random_graph(n_nodes = 5, edge_prob = 0.1)
)
central_graph <- compute_central_graph(Control)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compute_test_statistic.R
\name{compute_test_statistic}
\alias{compute_test_statistic}
\title{Compute the Test Statistic T for Brain Network Populations}
\usage{
compute_test_statistic(populations, a = 1)
}
\arguments{
\item{populations}{A named list where each element is a list of adjacency
matrices for a population. Example:
\code{list(Control = list(G1, G2, ...), PatientA = list(G1, G2, ...), PatientB = list(G1, G2, ...))}}

\item{a}{A normalization constant. Default is 1.}
}
\value{
A numeric value representing the test statistic T.
}
\description{
This function computes the test statistic T to assess whether different
populations of brain networks originate from the same distribution.
}
\examples{
# Generate synthetic populations data
Control <- list(
  generate_random_graph(n_nodes = 5, edge_prob = 0.1),
  generate_random_graph(n_nodes = 5, edge_prob = 0.1)
)
PatientA <- list(
  generate_random_graph(n_nodes = 5, edge_prob = 0.15),
  generate_random_graph(n_nodes = 5, edge_prob = 0.15)
)
PatientB <- list(
  generate_random_graph(n_nodes = 5, edge_prob = 0.2),
  generate_random_graph(n_nodes = 5, edge_prob = 0.2)
)

populations <- list(Control = Control, PatientA = PatientA, PatientB = PatientB)

# Compute the test statistic T
T_value <- compute_test_statistic(populations, a = 1)
print(T_value)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_critical_edges.R
\name{plot_critical_edges}
\alias{plot_critical_edges}
\title{Visualize Central Graphs and Critical Edges}
\usage{
plot_critical_edges(
  populations,
  critical_links,
  communities = NULL,
  layout = NULL,
  central_graphs = NULL,
  reference = 1L,
  threshold = 0.5,
  edge_scale = 6,
  critical_color = "red",
  critical_width = 3,
  background_color = "grey80",
  background_width = 1,
  vertex_size = 12,
  vertex_label = TRUE,
  panel_titles = NULL,
  mfrow = NULL,
  ...
)
}
\arguments{
\item{populations}{A named \code{list} of populations of adjacency matrices,
as accepted by \code{\link{identify_critical_links}}. Used to compute the
per-population central graphs unless \code{central_graphs} is supplied.}

\item{critical_links}{The object returned by
\code{\link{identify_critical_links}}. Its \code{critical_edges} component
provides the edges to highlight in the final panel.}

\item{communities}{Optional integer (or factor) vector of length
\code{n_nodes} giving the community membership of every node. Used to
color vertices consistently across panels. Default \code{NULL} (all
vertices share the default \pkg{igraph} color).}

\item{layout}{Optional layout for the graphs. Either an
\code{n_nodes x 2} numeric matrix of vertex coordinates (recommended, so
that all panels share the same node positions) or an \pkg{igraph} layout
function such as \code{\link[igraph]{layout_with_fr}}. Default
\code{\link[igraph]{layout_in_circle}}.}

\item{central_graphs}{Optional list of pre-computed central graphs (one per
population). If \code{NULL}, they are computed via
\code{\link{compute_central_graph}}.}

\item{reference}{Either an integer index or a name selecting which
population's central graph is used as the background of the
critical-edges panel. Default \code{1L} (the first population).}

\item{threshold}{Numeric in \code{[0, 1]}. The reference central graph is
binarised by \code{> threshold} for the critical-edges panel. Default
\code{0.5}.}

\item{edge_scale}{Numeric multiplier applied to the central-graph edge
weights to set \code{edge.width} in the per-population panels. Default
\code{6}.}

\item{critical_color, critical_width}{Color and line width used to draw
critical edges. Defaults \code{"red"} and \code{3}.}

\item{background_color, background_width}{Color and line width used to draw
non-critical edges in the critical panel. Defaults \code{"grey80"} and
\code{1}.}

\item{vertex_size}{Vertex size passed to \code{\link[igraph]{plot.igraph}}.
Default \code{12}.}

\item{vertex_label}{Logical. If \code{TRUE} (default) vertex indices are
shown; otherwise vertex labels are suppressed.}

\item{panel_titles}{Optional character vector of length
\code{length(populations) + 1} with custom panel titles. Default uses
\code{paste(names(populations), "Central Graph")} followed by
\code{"Critical edges"}.}

\item{mfrow}{Optional integer vector of length 2 giving the
\code{c(nrow, ncol)} arrangement passed to \code{\link[graphics]{par}}.
By default a near-square grid is chosen automatically (e.g.\ a
\code{2 x 2} grid for two populations, matching the standard
Control / Patient / Critical layout).}

\item{...}{Additional arguments forwarded to
\code{\link[igraph]{plot.igraph}}. Note that \code{edge.width},
\code{vertex.color}, \code{vertex.size}, \code{vertex.label} and
\code{layout} are managed by this function and should not be passed via
\code{...}.}
}
\value{
Invisibly returns \code{NULL}; called for its side effect of
  producing the multi-panel plot.
}
\description{
Produces a multi-panel figure summarising the output of
\code{\link{identify_critical_links}}: one panel per population showing the
weighted central graph, plus a final panel that highlights the critical
edges on a chosen reference central graph. This is the canonical
visualization of the \pkg{BrainNetTest} workflow and replaces the manual
\pkg{igraph} plumbing required to build the same figure.
}
\details{
The first \code{length(populations)} panels show the weighted
  central graph of each population, with edge widths proportional to the
  central-graph weights. The final panel binarises the chosen reference
  central graph at \code{threshold} and overlays the critical edges
  returned by \code{identify_critical_links()} in
  \code{critical_color}. Critical edges that are absent from the binarised
  reference graph are added so that they remain visible.

  The graphics state (\code{par(mfrow, mar, oma)}) is restored on exit.
}
\examples{
\donttest{
set.seed(123)
community_sizes <- c(4, 2, 3, 3, 5)
control <- generate_category_graphs(
  n_graphs = 50, n_nodes = 17, n_communities = 5,
  community_sizes = community_sizes,
  base_intra_prob = rep(0.70, 5), base_inter_prob = 0.05,
  intra_prob_variation = 0.02, inter_prob_variation = 0.01)
patient <- generate_category_graphs(
  n_graphs = 50, n_nodes = 17, n_communities = 5,
  community_sizes = community_sizes,
  base_intra_prob = c(0.40, 0.70, 0.70, 0.70, 0.70),
  base_inter_prob = 0.05,
  intra_prob_variation = 0.02, inter_prob_variation = 0.01)
populations <- list(Control = control, Patient = patient)
result <- identify_critical_links(populations, alpha = 0.05,
  method = "fisher", n_permutations = 200, seed = 1)
communities <- rep(seq_along(community_sizes), times = community_sizes)
plot_critical_edges(populations, result, communities = communities)
}
}
\seealso{
\code{\link{identify_critical_links}},
  \code{\link{compute_central_graph}},
  \code{\link{get_critical_nodes}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compute_edge_frequencies.R
\name{compute_edge_frequencies}
\alias{compute_edge_frequencies}
\title{Compute Edge Frequencies in Populations}
\usage{
compute_edge_frequencies(populations)
}
\arguments{
\item{populations}{A list where each element is a population containing a list of graphs (adjacency matrices).}
}
\value{
A list containing edge counts and edge proportions for each population.
}
\description{
This function computes the frequency of each edge (connection between nodes)
across all graphs within each population.
}
\examples{
# Generate synthetic populations
control_graphs <- generate_category_graphs(
  n_graphs = 5,
  n_nodes = 10,
  n_communities = 2,
  base_intra_prob = 0.8,
  base_inter_prob = 0.2,
  intra_prob_variation = 0.05,
  inter_prob_variation = 0.05,
  seed = 1
)
patient_graphs <- generate_category_graphs(
  n_graphs = 5,
  n_nodes = 10,
  n_communities = 2,
  base_intra_prob = 0.6,
  base_inter_prob = 0.4,
  intra_prob_variation = 0.05,
  inter_prob_variation = 0.05,
  seed = 2
)
populations <- list(Control = control_graphs, Patient = patient_graphs)

# Compute edge frequencies
frequencies <- compute_edge_frequencies(populations)
# View edge counts for the first population
edge_counts_control <- frequencies$edge_counts[,,1]
print(edge_counts_control)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compute_edge_pvalues.R
\name{compute_edge_pvalues}
\alias{compute_edge_pvalues}
\title{Compute P-Values for Edge Differences Between Populations}
\usage{
compute_edge_pvalues(edge_counts, N, method = "fisher", adjust_method = "none")
}
\arguments{
\item{edge_counts}{An array of edge counts from \code{compute_edge_frequencies}.}

\item{N}{A vector of sample sizes for each population.}

\item{method}{The statistical test method to use: "fisher", "chi.squared", or "prop". Default is "fisher".}

\item{adjust_method}{The method for p-value adjustment for multiple testing. Default is "none".}
}
\value{
A matrix of p-values for each edge.
}
\description{
This function computes p-values for the differences in edge presence between populations
using statistical tests (e.g., Fisher's exact test, chi-squared test).
}
\examples{
# Generate synthetic populations
control_graphs <- generate_category_graphs(
  n_graphs = 5,
  n_nodes = 10,
  n_communities = 2,
  base_intra_prob = 0.8,
  base_inter_prob = 0.2,
  intra_prob_variation = 0.05,
  inter_prob_variation = 0.05,
  seed = 1
)
patient_graphs <- generate_category_graphs(
  n_graphs = 5,
  n_nodes = 10,
  n_communities = 2,
  base_intra_prob = 0.6,
  base_inter_prob = 0.4,
  intra_prob_variation = 0.05,
  inter_prob_variation = 0.05,
  seed = 2
)
populations <- list(Control = control_graphs, Patient = patient_graphs)

# Compute edge frequencies
frequencies <- compute_edge_frequencies(populations)
edge_counts <- frequencies$edge_counts
N <- sapply(populations, length)

# Compute p-values for edge differences
edge_pvalues <- compute_edge_pvalues(edge_counts, N)
# View p-values for the first few edges
print(edge_pvalues[1:5, 1:5])
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_critical_nodes.R
\name{get_critical_nodes}
\alias{get_critical_nodes}
\title{Extract Critical Nodes from Critical Edge Results}
\usage{
get_critical_nodes(result, node_labels = NULL)
}
\arguments{
\item{result}{The list returned by \code{\link{identify_critical_links}}.
Must contain a \code{critical_edges} component (a data frame with columns
\code{node1} and \code{node2}, or \code{NULL}).}

\item{node_labels}{An optional character vector of length \eqn{p} (the number
of nodes in the network), where \code{node_labels[i]} is the label for
node \code{i}. If \code{NULL} (the default), no labels are included.}
}
\value{
A data frame with the following columns, ordered by decreasing
  \code{critical_degree}:
  \describe{
    \item{node}{Integer node index.}
    \item{label}{Character label for the node (only present when
      \code{node_labels} is not \code{NULL}).}
    \item{critical_degree}{Number of critical edges incident on this node.}
  }
  If no critical edges were found (\code{result$critical_edges} is
  \code{NULL} or has zero rows), an empty data frame with the same columns
  is returned.
}
\description{
Given the output of \code{\link{identify_critical_links}}, this function
identifies the unique nodes involved in the critical edges and summarizes
their participation. Each node is reported with the number of critical edges
it participates in (its \emph{critical degree}). If node labels are supplied,
they are included in the output.
}
\examples{
# Generate two synthetic populations with different community structure
control <- generate_category_graphs(n_graphs = 15, n_nodes = 10,
  n_communities = 2, base_intra_prob = 0.8, base_inter_prob = 0.2, seed = 1)
patient <- generate_category_graphs(n_graphs = 15, n_nodes = 10,
  n_communities = 2, base_intra_prob = 0.5, base_inter_prob = 0.5, seed = 2)
populations <- list(Control = control, Patient = patient)

# Identify critical edges
result <- identify_critical_links(populations, alpha = 0.05,
  method = "fisher", n_permutations = 200, seed = 42)

# Extract critical nodes (no labels)
get_critical_nodes(result)

# With labels
labels <- paste0("Region_", 1:10)
get_critical_nodes(result, node_labels = labels)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compute_distance.R
\name{compute_distance}
\alias{compute_distance}
\title{Compute the Manhattan Norm Distance Between Two Graphs}
\usage{
compute_distance(G, M)
}
\arguments{
\item{G}{A square adjacency matrix representing a brain network.}

\item{M}{A square adjacency matrix representing the central brain network.}
}
\value{
A numeric value representing the Manhattan distance between G and M.
}
\description{
This function calculates the Manhattan norm (L1 norm) distance between two
adjacency matrices representing brain networks.
}
\examples{
# Generate synthetic data
G1 <- generate_random_graph(n_nodes = 5, edge_prob = 0.1)
G2 <- generate_random_graph(n_nodes = 5, edge_prob = 0.1)
central_graph <- compute_central_graph(list(G1, G2))
distance <- compute_distance(G1, central_graph)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generate_category_graphs.R
\name{generate_category_graphs}
\alias{generate_category_graphs}
\title{Generate a Set of Graphs with Similar Community Structures for a Category}
\usage{
generate_category_graphs(
  n_graphs = 10,
  n_nodes = 100,
  n_communities = 4,
  community_sizes = NULL,
  base_intra_prob = 0.8,
  base_inter_prob = 0.2,
  intra_prob_variation = 0.05,
  inter_prob_variation = 0.05,
  seed = NULL
)
}
\arguments{
\item{n_graphs}{An integer specifying the number of graphs to generate. Default is 10.}

\item{n_nodes}{An integer specifying the total number of nodes (brain regions). Default is 100.}

\item{n_communities}{An integer specifying the number of communities. Default is 4.}

\item{community_sizes}{An integer vector specifying the sizes of each community.
If NULL, communities are of equal size. Default is NULL.}

\item{base_intra_prob}{A numeric value between 0 and 1, or a numeric vector
of length \code{n_communities}, specifying the base probability of an edge
existing between nodes within the same community. If a vector, each element
sets the base probability for the corresponding community. Default is 0.8.}

\item{base_inter_prob}{A numeric value between 0 and 1 specifying the base probability
of an edge existing between nodes from different communities. Default is 0.2.}

\item{intra_prob_variation}{A numeric value specifying the maximum variation to apply
to the intra-community probability for each graph. Default is 0.05.}

\item{inter_prob_variation}{A numeric value specifying the maximum variation to apply
to the inter-community probability for each graph. Default is 0.05.}

\item{seed}{An optional integer for setting the random seed to ensure reproducibility.
Default is NULL.}
}
\value{
A list of symmetric binary adjacency matrices with no self-loops, representing
  brain networks with similar community structures.
}
\description{
This function generates a list of adjacency matrices representing brain networks
belonging to the same category (e.g., Control group). The generated graphs have
similar community structures but vary slightly in their intra-community and
inter-community connection probabilities to reflect natural variability.
}
\examples{
# Generate a set of 5 graphs for the Control category
control_graphs <- generate_category_graphs(n_graphs = 5, n_nodes = 100, n_communities = 4,
                                           base_intra_prob = 0.8, base_inter_prob = 0.2)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generate_community_graph.R
\name{generate_community_graph}
\alias{generate_community_graph}
\title{Generate a Random Symmetric Adjacency Matrix with Community Structure}
\usage{
generate_community_graph(
  n_nodes = 100,
  n_communities = 4,
  community_sizes = NULL,
  intra_prob = 0.8,
  inter_prob = 0.2,
  seed = NULL
)
}
\arguments{
\item{n_nodes}{An integer specifying the total number of nodes (brain regions).
Default is 100.}

\item{n_communities}{An integer specifying the number of communities. Default is 4.}

\item{community_sizes}{An integer vector specifying the sizes of each community.
If NULL, communities are of equal size. Default is NULL.}

\item{intra_prob}{A numeric value between 0 and 1, or a numeric vector of
length \code{n_communities}, specifying the probability of an edge existing
between nodes within the same community. If a scalar, the same probability
is used for all communities. Default is 0.8.}

\item{inter_prob}{A numeric value between 0 and 1 specifying the probability
of an edge existing between nodes from different communities. Default is 0.2.}

\item{seed}{An optional integer for setting the random seed to ensure reproducibility.
Default is NULL.}
}
\value{
A symmetric binary adjacency matrix with no self-loops, representing
  a brain network with community structure.
}
\description{
This function generates a random symmetric adjacency matrix representing
a brain network with community structure. Nodes within the same community
have a higher probability of being connected compared to nodes from different
communities.
}
\examples{
# Generate a brain network with community structure
G <- generate_community_graph(n_nodes = 100, n_communities = 4, intra_prob = 0.8, inter_prob = 0.2)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rank_edges.R
\name{rank_edges}
\alias{rank_edges}
\title{Rank Edges Based on P-Values}
\usage{
rank_edges(edge_pvalues)
}
\arguments{
\item{edge_pvalues}{A square matrix of p-values for each edge, typically obtained from \code{compute_edge_pvalues}.}
}
\value{
A data frame with edges and their corresponding p-values, ordered from most significant.
}
\description{
This function ranks edges based on their p-values obtained from statistical tests,
ordering them from lowest to highest p-value (most to least significant).
}
\examples{
# Generate synthetic populations
control_graphs <- generate_category_graphs(
  n_graphs = 5,
  n_nodes = 10,
  n_communities = 2,
  base_intra_prob = 0.8,
  base_inter_prob = 0.2,
  seed = 1
)
patient_graphs <- generate_category_graphs(
  n_graphs = 5,
  n_nodes = 10,
  n_communities = 2,
  base_intra_prob = 0.6,
  base_inter_prob = 0.4,
  seed = 2
)
populations <- list(Control = control_graphs, Patient = patient_graphs)

# Compute edge frequencies
frequencies <- compute_edge_frequencies(populations)
edge_counts <- frequencies$edge_counts
N <- sapply(populations, length)

# Compute p-values for edge differences
edge_pvalues <- compute_edge_pvalues(edge_counts, N)

# Rank edges based on p-values
edge_df <- rank_edges(edge_pvalues)
# View the top ranked edges
head(edge_df)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generate_random_graph.R
\name{generate_random_graph}
\alias{generate_random_graph}
\title{Generate a Random Symmetric Adjacency Matrix}
\usage{
generate_random_graph(n_nodes, edge_prob = 0.1)
}
\arguments{
\item{n_nodes}{An integer specifying the number of nodes (brain regions).}

\item{edge_prob}{A numeric value between 0 and 1 specifying the probability
of an edge existing between any two nodes.}
}
\value{
A symmetric binary adjacency matrix with no self-loops.
}
\description{
This function generates a random symmetric adjacency matrix representing
a brain network. The adjacency matrix is binary, with edges present based
on a specified probability.
}
\examples{
G <- generate_random_graph(n_nodes = 10, edge_prob = 0.1)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/identify_critical_links.R
\name{identify_critical_links}
\alias{identify_critical_links}
\title{Identify Critical Edges That Explain Population Differences}
\usage{
identify_critical_links(
  populations,
  alpha = 0.05,
  method = "fisher",
  adjust_method = "none",
  batch_size = 1,
  n_permutations = 1000,
  a = 1,
  seed = NULL
)
}
\arguments{
\item{populations}{A named \code{list} with at least two elements. Each
element is itself a list of square, binary, symmetric adjacency matrices
(zero diagonal, no self-loops). All graphs must share the same number of
nodes.}

\item{alpha}{Numeric significance level used both for the global test and
for the iterative edge-removal stopping rule. Default \code{0.05}.}

\item{method}{Character string selecting the marginal edge test used to
rank candidate edges. One of \code{"fisher"} (default),
\code{"chi.squared"}, or \code{"prop"}. See
\code{\link{compute_edge_pvalues}}.}

\item{adjust_method}{Character string passed to \code{\link[stats]{p.adjust}}
for multiplicity correction of the marginal p-values. Default
\code{"none"}.}

\item{batch_size}{Positive integer giving the number of edges to drop per
iteration. Larger values speed up the procedure at the cost of coarser
resolution. Default \code{1}.}

\item{n_permutations}{Positive integer giving the number of permutation
replicates used to approximate the null distribution of T. Default
\code{1000}.}

\item{a}{Positive numeric normalisation constant for the test statistic T
(see \code{\link{compute_test_statistic}}). Default \code{1}.}

\item{seed}{Optional integer seed for reproducibility of the permutation
replicates. Default \code{NULL}.}
}
\value{
A \code{list} with three components: \code{critical_edges} (a
  \code{data.frame} with columns \code{node1}, \code{node2}, \code{p_value},
  listing the edges removed until the test became non-significant, ordered
  from most to least significant; \code{NULL} if the global test was not
  significant), \code{edges_removed} (a list of length-2 integer vectors
  giving the \code{(i, j)} indices of the removed edges, in removal order),
  and \code{modified_populations} (the input \code{populations} with the
  critical edges zeroed out in every graph).
}
\description{
Iteratively identifies the edges that drive the observed difference between
two or more populations of brain networks, using the permutation-based
L1-distance ANOVA test of Fraiman and Fraiman (2018). Edges are ranked by
their marginal p-value, then removed in batches until the global test is no
longer significant. The permutation null is re-evaluated incrementally via
a prefix-sum decomposition of T, which avoids recomputing the statistic
from scratch at every step.
}
\details{
The implementation exploits the fact that T decomposes as a sum of
  per-edge contributions \eqn{\Delta_e}. Removing an edge e is equivalent
  to subtracting \eqn{\Delta_e} from T, so prefix sums of the sorted
  \eqn{\Delta_e} values give the test statistic after any number of
  removals in O(1). The permutation null is built once via a single BLAS
  matrix multiplication and reused for every iteration, reducing the
  overall complexity from O(K * B * |E| * m) (naive) to O(B * |E| * m).

  Permutation replicates are generated by uniformly permuting the group
  labels across the pooled sample of all graphs, while preserving the
  original group sizes (sampling without replacement).
}
\examples{
\donttest{
set.seed(1)
control <- generate_category_graphs(
  n_graphs = 15, n_nodes = 10, n_communities = 2,
  base_intra_prob = 0.8, base_inter_prob = 0.2, seed = 1)
patient <- generate_category_graphs(
  n_graphs = 15, n_nodes = 10, n_communities = 2,
  base_intra_prob = 0.5, base_inter_prob = 0.5, seed = 2)
populations <- list(Control = control, Patient = patient)
result <- identify_critical_links(
  populations, alpha = 0.05, method = "fisher",
  n_permutations = 200, seed = 42)
head(result$critical_edges)
}
}
\references{
Fraiman, D. and Fraiman, R. (2018) An ANOVA approach for statistical
comparisons of brain networks. \emph{Scientific Reports}, 8, 4746.
\doi{10.1038/s41598-018-21688-0}.
}
\seealso{
\code{\link{compute_test_statistic}},
  \code{\link{compute_edge_pvalues}}, \code{\link{get_critical_nodes}}.
}
